% Class file for ``Using LaTeX for Administrative Work''
% Provided "as is" with no warranty. There's a lot of
% fudging to help the code work with admin.perl and with all the
% different PDF formats.
\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{admin}[2013/08/20 (NLCT)]

\RequirePackage{etoolbox}

\newif\ifscreen
\screenfalse

\newcommand*{\extra@stuff}{}

\newcommand{\dosummarymark}[1]{%
  \def\@summarymarker{#1}%
  \@onelevel@sanitize\@summarymarker
  \edef\do@writeout{%
     \noexpand\protected@write\noexpand\@auxout{}{%
       \string\summarymarker{\noexpand\thepage}{\@summarymarker}%
     }%
  }%
  \do@writeout
}

\newcommand*{\file}[1]{\texorpdfstring{\nolinkurl{#1}}{#1}}

% Default is report
% This uses 10pt normal font size as the text width is quite narrow
% and leads to too many overfull lines in displayed code.
\def\admin@loadsty{admin-a4paper}
\def\admin@ptsize{10pt}
\newcount\adminmedia
\def\ADMINreport{0}
\def\ADMINscreen{1}
\def\ADMINbook{2}
\adminmedia=\ADMINreport\relax
\PassOptionsToClass{chapterprefix=true,oneside,captions=tableheading,numbers=noenddot,bibliography=totoc}{scrbook}%

\DeclareOption{report}{%
  \screenfalse
  \def\admin@loadsty{admin-a4paper}%
  \def\admin@ptsize{10pt}%
  \adminmedia=\ADMINreport\relax
  \PassOptionsToClass{chapterprefix=true,oneside,captions=tableheading,numbers=noenddot,bibliography=totoc}{scrbook}%
}
\DeclareOption{html}{%
  \screenfalse
  \def\admin@loadsty{admin-html}%
  \def\admin@ptsize{12pt}%
  \adminmedia=\ADMINreport\relax
  \PassOptionsToClass{chapterprefix=true,oneside,captions=tableheading,numbers=noenddot}{scrbook}%
  \renewcommand*{\extra@stuff}{%
    \hypersetup{linkcolor=black,citecolor=black,urlcolor=black}%
    \renewcommand{\tableref}[1]{\tablename~\ref{##1}}%
    \renewcommand{\vfigureref}[1]{\figurename~\ref{##1}}%
    \renewcommand{\figureref}[1]{\figurename~\ref{##1}}%
    \renewcommand{\listingref}[1]{Listing~\ref{##1}}%
    \DeclareRobustCommand{\cdash}{\dash}%
    \DeclareRobustCommand{\optfmt}[1]{\texttt{##1}}%
  }%
  \renewcommand{\dosummarymark}[1]{}%
  \robustify\file
}
\DeclareOption{screen}{%
  \screentrue
  \def\admin@loadsty{admin-screen}%
  \def\admin@ptsize{8pt}
  \adminmedia=\ADMINscreen\relax
  \PassOptionsToClass{chapterprefix=false,oneside,headings=small,captions=tableheading,numbers=noenddot,bibliography=totoc}{scrbook}%
}
\DeclareOption{book}{%
  \screenfalse
  \def\admin@loadsty{admin-book}%
  \def\admin@ptsize{10pt}
  \adminmedia=\ADMINbook\relax
  \PassOptionsToClass{chapterprefix=true,twoside,headings=normal,captions=tableheading,numbers=noenddot,bibliography=totoc}{scrbook}%
  \PassOptionsToPackage{gray}{xcolor}%
}
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{scrbook}}

\ProcessOptions

\LoadClass{scrbook}

\expandafter\RequirePackage\expandafter[\admin@ptsize]{extsizes}

\RequirePackage{scrwfile}
\RequirePackage{keystroke}

\RequirePackage{graphicx}
\RequirePackage{amsmath}
\RequirePackage{upquote}
\RequirePackage{textcomp}
\RequirePackage{fancybox}
\RequirePackage{pifont}
\RequirePackage[figuresright]{rotating}

\renewcommand{\sidewaysfigure}{%
 \if@figurecaptionabove
  \let\caption\captionabove
 \else
  \let\caption\captionbelow
 \fi
 \@rotfloat{figure}%
}

\renewcommand{\sidewaystable}{%
 \if@tablecaptionabove
  \let\caption\captionabove
 \else
  \let\caption\captionbelow
 \fi
 \@rotfloat{table}%
}

\RequirePackage[x11names]{xcolor}

\RequirePackage{tikz}
\usetikzlibrary{arrows}
\usetikzlibrary{decorations.pathmorphing}
\usetikzlibrary{calc}
\usetikzlibrary{shapes.multipart}

\RequirePackage{ifsym}

\RequirePackage{pgfcalendar}

\RequirePackage{metalogo}

\RequirePackage{siunitx}

% eTeX logo (definition from etex manual)
\providecommand*{\eTeX}{\leavevmode\hbox{$\varepsilon$}-\TeX}

\DeclareSIUnit\inch{in}
\DeclareSIUnit\spunit{sp}
\DeclareSIUnit\ptunit{pt}
\DeclareSIUnit\exunit{ex}
\DeclareSIUnit\emunit{em}

% Requires at least version 1.08 of the flowfram package
\RequirePackage{\admin@loadsty}

\ifnum\adminmedia=\ADMINbook
 \setlength{\marginparwidth}{0.7in}
 \newlength\@even@offset
 \setlength{\@even@offset}{0.3in}

 \setflowframe*{main,header,left,leftshort}{margin=outer}

 \@flowframeid{right}
 \expandafter\addtolength\csname col@\romannumeral\ff@id @evenx\endcsname{\@even@offset}
 \@flowframeid{rightshort}
 \expandafter\addtolength\csname col@\romannumeral\ff@id @evenx\endcsname{\@even@offset}

 \@flowframeid{left}
 \expandafter\addtolength\csname col@\romannumeral\ff@id @evenx\endcsname{\@even@offset}
 \@flowframeid{leftshort}
 \expandafter\addtolength\csname col@\romannumeral\ff@id @evenx\endcsname{\@even@offset}

 \@flowframeid{main}
 \expandafter\addtolength\csname col@\romannumeral\ff@id @evenx\endcsname{\@even@offset}

 \@dynamicframeid{header}
 \expandafter\addtolength\csname @df@\romannumeral\ff@id @evenx\endcsname{\@even@offset}

 \@dynamicframeid{footer}
 \expandafter\addtolength\csname @df@\romannumeral\ff@id @evenx\endcsname{\@even@offset}

 \@dynamicframeid{indextab}
 \expandafter\computeleftedgeeven
   \csname @df@\romannumeral\ff@id @evenx\endcsname
% \expandafter\addtolength
%   \csname @df@\romannumeral\ff@id @evenx\endcsname
%   {3mm}%
 \expandafter\addtolength
   \csname @df@\romannumeral\ff@id @posx\endcsname
   {3mm}%

 \setdynamiccontents{\ff@id}{%
   \color{white}\LARGE\bfseries\scshape\centering
   \ifodd\c@page\relax
     \hspace*{-1ex}\rotatebox[origin=cb]{-90}{Index}%
   \else
     \hspace*{1ex}\rotatebox[origin=cb]{90}{Index}%
   \fi
   \par
 }

 \@dynamicframeid{summarytab}
 \expandafter\computeleftedgeeven
   \csname @df@\romannumeral\ff@id @evenx\endcsname
% \expandafter\addtolength
%   \csname @df@\romannumeral\ff@id @evenx\endcsname
%   {3mm}%
 \expandafter\addtolength
   \csname @df@\romannumeral\ff@id @posx\endcsname
   {3mm}%

 \setdynamiccontents{\ff@id}{%
   \color{white}\LARGE\bfseries\scshape\centering
   \ifodd\c@page\relax
     \hspace*{-1ex}\rotatebox[origin=cb]{-90}{Summary}%
   \else
     \hspace*{1ex}\rotatebox[origin=cb]{90}{Summary}%
   \fi
   \par
 }

\fi

\RequirePackage{pifont}
\RequirePackage{ifthen}
\RequirePackage{ifpdf}
\RequirePackage{url}
\RequirePackage{datetime}
\RequirePackage{makeidx}

\RequirePackage{datatool}

% html package conflicts with varioref+hyperref, so emulate it:
\RequirePackage{comment}
\specialcomment{htmlonly}{}{}
\excludecomment{htmlonly}
\newenvironment{latexonly}{\ignorespaces}{\ignorespacesafterend}
\newenvironment{makeimage}{}{}
\newcommand{\latex}[1]{#1}
\newcommand{\latexhtml}[2]{#1}
\newcommand{\html}[1]{}
\newcommand{\htmlimage}[2]{}
\newcommand{\mathorit}[1]{$#1$}
\newcommand{\mathorrm}[1]{$#1$}

% LaTeX2HTML doesn't understand the starred version of \\
\newcommand*{\nobrcr}{\tabularnewline*}

\RequirePackage{varioref}
\RequirePackage{xr-hyper}
\RequirePackage[hyperindex=false]{hyperref}
\RequirePackage
  [
   toc,
   subentrycounter,
   sanitizesort=false,
   acronym,
   indexonlyfirst,
   counter=section,
   numberedsection=nolabel,
   savewrites
  ]%
  {glossaries-accsupp}
\RequirePackage{glossary-inline}

\newcommand*{\suppresslevels}{%
  \let\basiclevel\relax
  \let\mediumlevel\relax
  \let\hardlevel\relax
}

\pdfstringdefDisableCommands{\suppresslevels}

% Redefine \htmladdnormallinkfoot so that it typesets the url in
% typewriter font for the book, but does not footnote for report and
% screen versions:

\ifnum\adminmedia=\ADMINbook\relax
  \renewcommand*{\htmladdnormallinkfoot}[2]{%
    \@ifnextpunc{\@linkfoot{#1}{#2}}{\@linkfoot{#1}{#2}{}}%
  }
  \newcommand*{\@linkfoot}[3]{%
    \href{#2}{#1#3}\footnote{\file{#2}}%
  }
  \newcommand*{\textlinkorurl}[2]{\url{#2}}
\else
  \renewcommand*{\htmladdnormallinkfoot}[2]{\href{#2}{#1}}
  \newcommand*{\textlinkorurl}[2]{\href{#2}{#1}}
\fi

% don't check for makeindex special characters, as they're dealt
% with explicitly
\let\@gls@checkmkidxchars\@gobble

\longdate


\ifnum\adminmedia=\ADMINbook
  \hypersetup{draft}
\else
  \hypersetup{colorlinks,plainpages=false,
  linkcolor=blue,menucolor=blue,citecolor=blue,
  urlcolor=magenta,filecolor=magenta}
\fi

\newcommand{\vfigureref}[1]{\figurename~\vref{#1}}

\ifnum\adminmedia=\ADMINbook
 \newcommand{\figureref}[1]{\figurename~\vref{#1}}
 \newcommand{\tableref}[1]{\tablename~\vref{#1}}
 \newcommand{\listingref}[1]{Listing~\vref{#1}}
\else
  \newcommand{\figureref}[1]{\hyperlink{#1top}{\figurename~\ref*{#1}}}
  \newcommand{\tableref}[1]{\htmlref{\tablename~\ref*{#1}}{#1}}
  \newcommand{\listingref}[1]{\htmlref{Listing~\ref*{#1}}{#1}}
\fi

\extra@stuff

\robustify\htmlref

\let\jdrheadingcase\@firstofone

\renewcommand*{\headfont}{%
 \let\basiclevel\relax\let\mediumlevel\relax\let\hardlevel\relax
 \em
}

\ifnum\adminmedia=\ADMINscreen\relax
  \renewcommand*{\footfont}{\small\scshape}
  \renewcommand*{\fps@figure}{!htbp}
\else
  \renewcommand*{\footfont}{\em}
  \renewcommand*{\fps@figure}{htbp}
\fi

\newcommand*{\leftfootmark}{}
\newcommand*{\rightfootmark}{}
\renewcommand*{\ps@headings}{%
  \let\@mkboth\markboth
  \renewcommand*{\@evenhead}{\normalfont\headfont\thepage\hfill\leftmark}%
  \renewcommand*{\@oddhead}{\normalfont\headfont\rightmark\hfill\thepage}%
  \renewcommand*{\@evenfoot}{\normalfont\footfont\leftfootmark\@leftfoothook\hfill}%
  \renewcommand*{\@oddfoot}{\hfill\normalfont\footfont\rightfootmark\@rightfoothook}%
}

\renewcommand*{\ps@plain}{%
  \renewcommand*{\@oddhead}{}%
  \renewcommand*{\@evenhead}{}%
  \renewcommand*{\@evenfoot}{\normalfont\footfont\thepage\hfill}%
  \renewcommand*{\@oddfoot}{\normalfont\footfont\hfill\thepage}%
}

\pagestyle{headings}

\newcommand*{\@leftfoothook}{}
\newcommand*{\@rightfoothook}{}

\newcommand*{\setleftfootmark}[1]{}

\newcommand*{\@setleftfootmark}[1]{%
  \gdef\leftfootmark{#1}%
  \global\let\setleftfootmark\@gobble
}

\newcommand*{\setrightfootmark}[1]{}

\newcommand*{\@setrightfootmark}[1]{%
  \gdef\rightfootmark{#1}%
}

\setdynamicframe*{footer}{hide=false}%

\ifnum\adminmedia=\ADMINbook
  \newcommand*{\enablefoothooks}{%
    \let\setrightfootmark\@setrightfootmark
    \let\setleftfootmark\@setleftfootmark
    \renewcommand*{\@rightfoothook}{%
      \global\let\setleftfootmark\@setleftfootmark
    }%
    \renewcommand*{\@leftfoothook}{%
      \global\let\setleftfootmark\@setleftfootmark
    }%
  }
\else
  \newcommand*{\enablefoothooks}{}
\fi

\newcommand*{\disablefoothooks}{%
  \global\let\leftfootmark\@empty
  \global\let\rightfootmark\@empty
  \global\let\setrightfootmark\@gobble
  \global\let\setleftfootmark\@gobble
  \global\let\@leftfoothook\@empty
  \global\let\@rightfoothook\@empty
}

\providecommand{\abstractname}{Abstract}

\newcommand*{\@edition}{}
\newcommand*{\edition}[1]{\renewcommand*{\@edition}{#1}}
\newcommand*{\theversion}{}
\newcommand*{\version}[1]{\renewcommand*{\theversion}{#1}}
\newcommand*{\thevolume}{}
\newcommand*{\volume}[1]{\renewcommand*{\thevolume}{#1}}
\newcommand*{\theseries}{}
\newcommand*{\series}[1]{\renewcommand*{\theseries}{#1}}

\newcommand*{\theseriesurl}{}
\newcommand*{\seriesurl}[1]{\renewcommand*{\theseriesurl}{#1}}

\newcommand{\navigationbar}{{%
\ifthenelse{\equal{\thepage}{a}}{\textcolor[gray]{.75}{Previous}}{%
\Acrobatmenu{PrevPage}{Previous}}\enspace
\ifthenelse{\boolean{lastpg}}{\textcolor[gray]{.75}{Next}}{%
\Acrobatmenu{NextPage}{Next}}\enspace
\Acrobatmenu{FirstPage}{First}\enspace
\Acrobatmenu{LastPage}{Last}\enspace
\Acrobatmenu{GoBack}{Back}\enspace
\Acrobatmenu{GoForward}{Forward}\enspace
\htmlref{Summary}{ch:glossary}\enspace
\htmlref{Index}{ch:index}}}

\newcommand{\ps@online}{%
\renewcommand{\@oddhead}{{\normalfont\headfont\rightmark}\hfil}%
\renewcommand{\@evenhead}{{\normalfont\hfil\headfont\leftmark}}%
\renewcommand{\@oddfoot}{\normalfont\footfont\hfill
\navigationbar
\hfill\thepage}
\renewcommand{\@evenfoot}{\@oddfoot}}


\ifscreen
\pagestyle{online}

\renewcommand{\ps@plain}{%
\renewcommand{\@oddhead}{}%
\renewcommand{\@evenhead}{}%
\renewcommand{\@oddfoot}{\normalfont\footfont\hfill
\navigationbar
\hfill\thepage}
\renewcommand{\@evenfoot}{\@oddfoot}}

\renewcommand{\ps@empty}{%
\renewcommand{\@oddhead}{}%
\renewcommand{\@evenhead}{}%
\renewcommand{\@oddfoot}{\normalfont\footfont\hfill
\navigationbar \hfill\mbox{}}
\renewcommand{\@evenfoot}{\@oddfoot}}

\fi

\ifnum\adminmedia=\ADMINscreen\relax
  \renewcommand*{\chapterformat}{%
  \chapappifchapterprefix{\ }\textup{\thechapter}.\enspace}
  \addtokomafont{chapter}{\centering\rmfamily\scshape}
\else
  \renewcommand*{\chapterformat}{%
  \chapappifchapterprefix{\ }\thechapter}
  \addtokomafont{chapter}{\raggedleft\rmfamily\scshape}
\fi

\addtokomafont{chapterentry}{\large\rmfamily\scshape}
\addtokomafont{chapterentrypagenumber}{\large\rmfamily\upshape}
\addtokomafont{section}{\rmfamily\bfseries}
\addtokomafont{subsection}{\rmfamily\bfseries}
\addtokomafont{subsubsection}{\rmfamily\bfseries}
\addtokomafont{minisec}{\rmfamily\bfseries\scshape}

\ifnum\adminmedia=\ADMINreport
  \renewcommand{\chapterheadstartvskip}{%
    \vspace*{1.5\baselineskip}%
  }
\fi

\renewcommand*\l@chapter[2]{%
  \ifnum\c@tocdepth>\m@ne
    \ifnum\lastpenalty<20010
      \addpenalty{-\@highpenalty}%
    \fi
    \vskip 1.0em \@plus \p@ 
    \setlength \@tempdima{2em}%
    \if@tocleft
      \ifx\toc@l@number\@empty
      \else
        \setlength \@tempdima{0\toc@l@number}%
      \fi
    \fi 
    \begingroup
      \raggedchapterentry
      \parindent\z@
      \advance\rightskip\@pnumwidth 
      \parfillskip -\@pnumwidth
      \interlinepenalty\@M
      \leavevmode
      \advance\leftskip \@tempdima
      \null\nobreak
      \hskip -\leftskip
      \usekomafont{chapterentry}%
      {%
        #1\nobreak
        \usekomafont{chapterentrypagenumber}%
        {%
          \hfill
          \nobreak
          \hb@xt@\@pnumwidth{\hss #2}%
        }%
      }%
      \par
      \ifnum\scr@compatibility>\@nameuse{scr@v@2.96}%
       \relax
       \endgroup
       \penalty 20009
     \else
       \penalty\@highpenalty
       \endgroup
     \fi
   \fi
}

\renewcommand*{\l@section}{%
  \bprot@dottedtocline {1}{1.5em}{3em}%
}

\renewcommand*{\captionformat}{\space}

\renewcommand*{\l@figure}{%
  \@dottedtocline {1}{1.5em}{3em}%
}

\renewcommand{\title}[2][]{\ifthenelse{\equal{#1}{}}{%
\renewcommand{\@pdftitle}{#2}}{
\ifthenelse{\equal{\@pdftitle}{}}{\renewcommand{\@pdftitle}{#1}}{}}%
\renewcommand{\@title}{#2}}
\let\booktitlenewline\relax

\renewcommand{\author}[2][]{\ifthenelse{\equal{#1}{}}{%
\ifthenelse{\equal{\@pdfauthor}{}}{%
\renewcommand{\@pdfauthor}{#2}}{}}{\renewcommand{\@pdfauthor}{#1}}%
\renewcommand{\@author}{#2}}

\newcommand{\keywords}[1]{\renewcommand{\@pdfkeywords}{#1}}

\providecommand{\subject}[1]{%
\renewcommand{\@pdfsubject}{#1}}

\newcommand*{\@affiliation}{}
\newcommand*{\thepublisher}{\@affiliation}
\newcommand*{\@affiliationurl}{}
\newcommand*{\@affiliationaddress}{}

\newcommand*{\affiliation}[3]{%
  \renewcommand{\@affiliation}{#1}%
  \renewcommand{\@affiliationurl}{\url{#2}}%
  \renewcommand{\@affiliationaddress}{#3}%
}

\ifpdf
 \DeclareGraphicsExtensions{.pdf,.png,.jpg}
\else
 \DeclareGraphicsExtensions{.ps,.eps}
\fi

\ifnum\adminmedia=\ADMINbook
  % book doesn't have sidepanel
\else
  % get the dimensions of sidepanel (values are stored in \ffareax, \ffareay, \ffareawidth and \ffareaheight)
  \getdynamicbounds*{sidepanel}

  % set the margin width to be the same as the width of sidepanel
  \setlength{\marginparwidth}{\ffareawidth}
\fi

\newcommand*{\index@preamble}{}
\newcommand{\indexpreamble}[1]{%
  \renewcommand{\index@preamble}{#1}%
}

% renew theindex environment so that it appears in the table of contents
% and set up index frames

\renewcommand{\@idxitem}{\par \hangindent 4em}

\renewenvironment{theindex}
{%
  \clearpage
  \begingroup
   \normalfont\sectfont
   \chapterheadstartvskip
   \phantomsection
   \size@chapter{\indexname}%
   \thispagestyle{plain}%
   \def\@currentlabelname{\indexname}%
   \label{ch:index}%
   \par
  \endgroup
  \markboth{\indexname}{\indexname}%
  \addcontentsline{toc}{chapter}{\indexname}%
  \parindent\z@
  \index@preamble\par
  \enablefoothooks
  \parskip\z@ \@plus .3\p@\relax
  \let\item\@idxitem
  \ifnum\adminmedia=\ADMINbook\relax
    \setleftfootmark{\doindextopmark}%
    \setrightfootmark{\doindexbottommark}%
  \else
    \setdynamiccontents*{sidepanel}{%
      \hfil\ifscreen\normalsize\else\large\fi\indexbar\hfil}%
  \fi
  \ifscreen\small\fi
  \newpage
  \ignorespacesafterend
}
{%
  \protected@write\@auxout{}{\string\@flow@endindex{\theabsolutepage}}%
  \protected@write\@auxout{}{\string\@flow@old@last@idx@page{\number\@last@idx@page}}%
  \setrightfootmark{\doindexbottommark\disablefoothooks}%
  \setleftfootmark{\doindextopmark\disablefoothooks}%
}

\newcount\@first@idx@page
\newcount\@last@idx@page
\newcommand*{\@flow@endindex}[1]{\global\@last@idx@page=#1\relax}

\newcommand*{\@old@last@idx@page}{-1}
\newcommand*{\@flow@old@last@idx@page}[1]{%
  \gdef\@old@last@idx@page{#1}%
}

\newcount\@last@summary@page
\newcommand*{\@flow@endsummary}[1]{%
  \global\@last@summary@page=#1\relax
  \global\@first@idx@page=\@last@summary@page
  \global\advance\@first@idx@page by 1\relax
  \ifnum\adminmedia=\ADMINbook\relax
    \ifodd\@first@idx@page
      \relax
    \else
      \global\advance\@first@idx@page by 1\relax
    \fi
  \fi
}

\newcount\@first@summary@page

\newcount\@last@acr@page
\newcommand*{\@flow@endacr}[1]{\global\@last@acr@page=#1\relax}

\newcommand*{\@flow@startsummary}[1]{%
  \global\@first@summary@page=#1\relax
   \ifnum\adminmedia=\ADMINbook\relax
     \ifodd\@first@summary@page
       \relax
     \else
       \global\advance\@first@summary@page by 1\relax
    \fi
  \fi
}


% Put glossary in double-column

\newcommand*{\setdoublecolumnglo}{%
  \let\glossarysection\dblglossarysection
}

\newcount\tmp@count

\newcommand{\@setup@pages}{%
  \ifnum\adminmedia=\ADMINbook\relax
    \setdynamicframe*{summarytab}{pages={\number\@first@summary@page-\number\@last@summary@page}}%
    \setdynamicframe*{indextab}{pages={\number\@first@idx@page-\number\@last@idx@page}}%
  \else
    \setdynamicframe*{sidepanel}%
      {pages={\number\@first@summary@page-\number\@last@summary@page,%
              \number\@first@idx@page-\number\@last@idx@page}}%
  \fi
  \ifnum\@first@summary@page>0\relax
    \@ff@tmpN=\@first@summary@page
    \advance\@ff@tmpN by 1\relax
    \tmp@count=\@first@idx@page
    \advance\tmp@count by 1\relax
    \edef\@nv@set@flf{\noexpand\setflowframe*{left,right}%
      {pages={\number\@ff@tmpN-\number\@last@summary@page
             ,\number\tmp@count-\number\@last@idx@page}}}%
    \@nv@set@flf
    \setflowframe*{main}%
      {pages={2-\number\@last@acr@page,\number\@last@acr@page,>\number\@last@idx@page}}
    \setflowframe*{summaryheader,summaryleftshort,summaryrightshort}%
      {pages={\number\@first@summary@page}}
    \setflowframe*{header,leftshort,rightshort}%
      {pages={\number\@first@idx@page}}
  \fi
}

\AtBeginDocument{\@setup@pages}

\newcommand*{\dblglossarysection}[2][\@gls@title]{%
  \doifnotbook
  {%
    \setdynamiccontents*{sidepanel}{%
      \hfil\ifscreen\normalsize\else\large\fi\summarybar\hfil}%
  }%
  \clearpage
  \protected@write\@auxout{}{\string\@flow@startsummary{\theabsolutepage}}%
  \begingroup
  \normalfont\sectfont
  \chapterheadstartvskip
  \phantomsection
  \size@chapter{#2}%
  \thispagestyle{plain}%
  \def\@currentlabelname{\glossarytoctitle}%
  \label{ch:glossary}%
  \par
  \endgroup
  \markboth{\glossarytoctitle}{\glossarytoctitle}%
  \addcontentsline{toc}{chapter}{#1}%
  \vskip\baselineskip
  \noindent
  \summarypreamble\par
  \newpage
}

\def\summarypreamble{}

\newif\iflastpg
\AtEndDocument
{%
  \lastpgtrue
  \ifnum\@last@idx@page=\@old@last@idx@page
    \relax
  \else
    \ClassWarningNoLine{admin}{Rerun required to get frames up-to-date}%
  \fi
}

\renewcommand{\maketitle}{%
  \pagenumbering{alph}% for PDF bookmarks
  \ifpdf
    \pdfinfo
    {%
       /Author (\@pdfauthor)
       /Title  (\@pdftitle)
       /Keywords (\@pdfkeywords)
       /Subject  (\@pdfsubject)
       /ModDate (D:\pdfdate)
       /CreationDate (D:20130820121402+01'00')
    }%
  \fi
  \begin{titlepage}%
   \doifbook{\let\booktitlenewline\@@par}%
   \@maketitle
   \let\booktitlenewline\relax
  \end{titlepage}%
  \thispagestyle{empty}%
  \clearpage
  \bgroup
    \thecopyrighttext
    \par
  \egroup
  \setcounter{footnote}{0}%
  \global\let\thetitle\@title
  \global\let\theauthor\@author
  \global\let\thanks\relax
  \global\let\maketitle\relax
  \global\let\@thanks\@empty
  \global\let\@author\@empty
  \global\let\@date\@empty
  \global\let\@title\@empty
  \global\let\title\relax
  \global\let\author\relax
  \global\let\date\relax
  \global\let\and\relax
  \normalfont
}

\providecommand{\theauthoryear}{}
\providecommand{\theauthorsurname}{}
\providecommand{\theauthorforenames}{}

\newcommand*{\authordetails}[3]{%
  \renewcommand*{\theauthoryear}{#1}%
  \renewcommand*{\theauthorsurname}{#2}%
  \renewcommand*{\theauthorforenames}{#3}%
  \renewcommand*{\@author}{#3\ #2}%
}

\newcommand\thetitle{\@title}

\providecommand*{\maketitlehook}{}

\providecommand{\thecopyrighttext}{}

\newcommand{\copyrighttext}[1]{%
  \appto\thecopyrighttext{#1}%
}

\ifcase\adminmedia

  % report version
  \newcommand{\indexbar}{%
    \begin{tabular}[t]{|c}
      \idxheaderlink{Symbols} \\
      \idxheaderlink{A} \\
      \idxheaderlink{B} \\
      \idxheaderlink{C} \\
      \idxheaderlink{D} \\
      \idxheaderlink{E} \\
      \idxheaderlink{F} \\
      \idxheaderlink{G} \\
      \idxheaderlink{H} \\
      \idxheaderlink{I} \\
      \idxheaderlink{J} \\
      \idxheaderlink{K} \\
      \idxheaderlink{L} \\
      \idxheaderlink{M} \\
      \idxheaderlink{N} \\
      \idxheaderlink{O} \\
      \idxheaderlink{P} \\
      \idxheaderlink{Q} \\
      \idxheaderlink{R} \\
      \idxheaderlink{S} \\
      \idxheaderlink{T} \\
      \idxheaderlink{U} \\
      \idxheaderlink{V} \\
      \idxheaderlink{W} \\
      \idxheaderlink{X} \\
      \idxheaderlink{Y} \\
      \idxheaderlink{Z}
    \end{tabular}%
  }

  \newcommand{\summarybar}{%
    \begin{tabular}[t]{|c}
      \glsheaderlink{glssymbols} \\
      \glsheaderlink{A} \\
      \glsheaderlink{B} \\
      \glsheaderlink{C} \\
      \glsheaderlink{D} \\
      \glsheaderlink{E} \\
      \glsheaderlink{F} \\
      \glsheaderlink{G} \\
      \glsheaderlink{H} \\
      \glsheaderlink{I} \\
      \glsheaderlink{J} \\
      \glsheaderlink{K} \\
      \glsheaderlink{L} \\
      \glsheaderlink{M} \\
      \glsheaderlink{N} \\
      \glsheaderlink{O} \\
      \glsheaderlink{P} \\
      \glsheaderlink{Q} \\
      \glsheaderlink{R} \\
      \glsheaderlink{S} \\
      \glsheaderlink{T} \\
      \glsheaderlink{U} \\
      \glsheaderlink{V} \\
      \glsheaderlink{W} \\
      \glsheaderlink{X} \\
      \glsheaderlink{Y} \\
      \glsheaderlink{Z}
    \end{tabular}%
  }

  \renewcommand{\@maketitle}{%
  \let\footnotesize\small
  \let\footnoterule\relax
  \let \footnote \thanks
  \null\vfil
  \vskip 60\p@
  \begin{center}%
    {\LARGE \@title \par}%
    \vskip 2ex%
    {\large Version \theversion\par}%
    \vskip 3em%
    {\large
     \lineskip .75em%
      \begin{tabular}[t]{c}%
        \@author
      \end{tabular}\par}%
      \vskip 1.5em%
    {\large 
       \begin{tabular}[t]{c}
       \@affiliation\\
       \@affiliationurl
       \end{tabular}\par}%
      \vskip 1.5em%
    {\large \@date \par}%       % Set date in \large size.
  \end{center}\par
  \@thanks
  \vfil\null
}

\or
  % screen version
  \newcommand{\indexbar}{%
    \begin{tabular}[t]{|cc}
    \multicolumn{2}{|c}{\idxheaderlink{Symbols}}\\
    \idxheaderlink{A} &
    \idxheaderlink{N} \\
    \idxheaderlink{B} &
    \idxheaderlink{O} \\
    \idxheaderlink{C} &
    \idxheaderlink{P} \\
    \idxheaderlink{D} &
    \idxheaderlink{Q} \\
    \idxheaderlink{E} &
    \idxheaderlink{R} \\
    \idxheaderlink{F} &
    \idxheaderlink{S} \\
    \idxheaderlink{G} &
    \idxheaderlink{T} \\
    \idxheaderlink{H} &
    \idxheaderlink{U} \\
    \idxheaderlink{I} &
    \idxheaderlink{V} \\
    \idxheaderlink{J} &
    \idxheaderlink{W} \\
    \idxheaderlink{K} &
    \idxheaderlink{X} \\
    \idxheaderlink{L} &
    \idxheaderlink{Y} \\
    \idxheaderlink{M} &
    \idxheaderlink{Z}
    \end{tabular}%
  }

  \newcommand{\summarybar}{%
    \begin{tabular}[t]{|cc}
    \multicolumn{2}{|c}{\glsheaderlink{glssymbols}}\\
    \glsheaderlink{A} &
    \glsheaderlink{N} \\
    \glsheaderlink{B} &
    \glsheaderlink{O} \\
    \glsheaderlink{C} &
    \glsheaderlink{P} \\
    \glsheaderlink{D} &
    \glsheaderlink{Q} \\
    \glsheaderlink{E} &
    \glsheaderlink{R} \\
    \glsheaderlink{F} &
    \glsheaderlink{S} \\
    \glsheaderlink{G} &
    \glsheaderlink{T} \\
    \glsheaderlink{H} &
    \glsheaderlink{U} \\
    \glsheaderlink{I} &
    \glsheaderlink{V} \\
    \glsheaderlink{J} &
    \glsheaderlink{W} \\
    \glsheaderlink{K} &
    \glsheaderlink{X} \\
    \glsheaderlink{L} &
    \glsheaderlink{Y} \\
    \glsheaderlink{M} &
    \glsheaderlink{Z}
    \end{tabular}%
  }

  \renewcommand\@maketitle{%
  \let\footnotesize\small
  \let\footnoterule\relax
  \let \footnote \thanks
  \null\vfil
  \begin{center}%
    {\LARGE \@title \par}%
    \vskip 2ex%
    {\large Version \theversion\par}%
    \vskip 3em%
    {\large
     \lineskip .75em%
      \begin{tabular}[t]{c}%
        \@author
      \end{tabular}\par}%
      \vskip 1.5em%
      {\large
       \begin{tabular}[t]{c}
       \@affiliation\\
       \@affiliationurl
       \end{tabular}\par}%
      \vskip 1.5em%
    {\large \@date \par}%       % Set date in \large size.
  \end{center}\par
  \@thanks
  \vfil\null
}

\or
 % book version

 \let\indexbar\relax
 \let\summarybar\relax

 \renewcommand\@maketitle{%
   \null\par
   \vskip4ex
   \centering
   \bfseries\scshape
   \Huge
   \@title
   \vskip4ex
   \Large
   \@author
   \par
   \vfill
   \normalsize
   Volume~\thevolume~of\par
   \theseries
   \par
   \vfill
   \maketitlehook
   \Large
   \mdseries
   \vskip2ex
   \@affiliation
   \par
   \vskip1ex
   \small
   \@affiliationaddress
   \par
   \vskip4ex
   \par\null
 }

\fi

\newlength\indexheaderskip
\newlength\indexheaderpreskip

\ifnum\adminmedia=\ADMINscreen
 \setlength{\indexheaderskip}{1ex}
 \setlength{\indexheaderpreskip}{1ex}
\else
 \setlength{\indexheaderskip}{1ex plus .5ex}
 \setlength{\indexheaderpreskip}{3.5ex plus .2ex minus .1ex}
\fi

\newcommand{\indexheader}[1]{%
 \par\noindent
 \vskip\indexheaderpreskip
 \par\noindent
 \makebox[\linewidth][c]%
 {%
   \ifnum\adminmedia=\ADMINbook\relax
     \textbf{#1}%
   \else
     \hypertarget{idx:#1}{\textbf{#1}}%
     \protected@write\@auxout{}{\string\idx@header{#1}}%
   \fi
 }%
 \par
 \nobreak
 \vskip\indexheaderskip
 \nobreak
}


\newcommand*{\idx@header}[1]{%
  \csgdef{idx@header@#1}{\hyperlink{idx:#1}{\textbf{#1}}}%
}

\newcommand*{\gls@header}[1]{%
  \csgdef{gls@header@#1}{\hyperlink{gls:#1}{\textbf{\glsgetgrouptitle{#1}}}}%
}

\newcommand*{\idxheaderlink}[1]{%
  \ifcsdef{idx@header@#1}%
  {\csname idx@header@#1\endcsname}%
  {\textbf{\color{gray}#1}}%
}

\newcommand*{\glsheaderlink}[1]{%
  \ifcsdef{gls@header@#1}%
  {\csname gls@header@#1\endcsname}%
  {\textbf{\color{gray}\glsgetgrouptitle{#1}}}%
}

\newtoks\indexmarktok

\newcommand{\doindexmark}[1]{%
  \def\@indexmarker{#1}%
  \@onelevel@sanitize\@indexmarker
  \edef\do@writeout{%
     \noexpand\protected@write\noexpand\@auxout{}{%
       \string\indexmarker{\noexpand\thepage}{\@indexmarker}%
     }%
  }%
  \do@writeout
}

\newcommand{\indexmarker}[2]{%
  \ifcsundef{@index@top@marker@\romannumeral#1}%
  {\csgdef{@index@top@marker@\romannumeral#1}{#2}}%
  {}%
  \csgdef{@index@bottom@marker@\romannumeral#1}{#2}%
}

\newcommand*{\doindextopmark}{%
  \csname @index@top@marker@\romannumeral\c@page\endcsname
}

\newcommand*{\doindexbottommark}{%
  \csname @index@bottom@marker@\romannumeral\c@page\endcsname
}

\newcommand{\summarymarker}[2]{%
  \ifcsundef{@summary@top@marker@\romannumeral#1}%
  {\csgdef{@summary@top@marker@\romannumeral#1}{#2}}%
  {}%
  \csgdef{@summary@bottom@marker@\romannumeral#1}{#2}%
}

\newcommand*{\dosummarytopmark}{%
  \csname @summary@top@marker@\romannumeral\c@page\endcsname
}

\newcommand*{\dosummarybottommark}{%
  \csname @summary@bottom@marker@\romannumeral\c@page\endcsname
}

\newcommand{\mainindexitem}[2]{\item #1\space\hfill#2\par}

\newcommand{\subindexitem}[2]{\subitem #1\hfill #2\par}
\newcommand{\subsubindexitem}[2]{\subsubitem #1\hfill #2\par}

% put headers and footers into the dynamic frames labelled
% 'header' and 'footer'
\renewcommand{\@dothehead}{}%
\renewcommand{\@dothefoot}{}%
\renewcommand{\@dodynamicthehead}{}
\renewcommand{\@dodynamicthefoot}{}

\ifnum\adminmedia=\ADMINbook\relax
  \setdynamiccontents*{header}{\vfill\ifodd\c@page\relax\@oddhead\else\@evenhead\fi\vfill}
  \setdynamiccontents*{footer}{\vfill\ifodd\c@page\relax\@oddfoot\else\@evenfoot\fi\vfill}
\else
  \setdynamiccontents*{header}{\vfill\@oddhead\vfill}
  \setdynamiccontents*{footer}{\vfill\@oddfoot\vfill}
\fi

\newcommand{\@frontmatter}{\mbox{}\par\vfill\framebreak\clearpage
\pagenumbering{roman}%
\setflowframe*{title}{pages=none}}

\@ifundefined{frontmatter}
 {%
   \let\frontmatter\@frontmatter
 }
 {%
   \let\orgfrontmatter\frontmatter
   \def\frontmatter{\@frontmatter\orgfrontmatter}
 }

\newcommand*{\@html@nodename}{\jobname}
\newcommand*{\setnode}[1]{\renewcommand*{\@html@nodename}{#1}}

\newcommand{\separator}{%
\begin{tikzpicture}
\fill (0,0) ..controls ++(1.74,0) and ++(0,0.1) .. (2,0)
  ..controls ++(0,-0.1) and ++(1.74,0) .. (0,0) -- cycle;
\fill (2.1,0) circle (0.065);
\fill (4.2,0) ..controls ++(-1.74,0) and ++(0,0.1) .. (2.2,0)
  ..controls ++(0,-0.1) and ++(-1.74,0) .. (4.2,0) -- cycle;
\end{tikzpicture}%
}

\newwrite\tf@hloe

\newcommand{\listofexamples}{%
  \chapter*{List of Examples}
  \addcontentsline{toc}{chapter}{List of Examples}%
  \markboth{List of Examples}{List of Examples}%
  \@starttoc{loe}
  \immediate\openout\tf@hloe \jobname.hloe\relax
  \immediate\write\tf@hloe{\string\begin{examplelist}}%
}

\AtEndDocument{\immediate\write\tf@hloe{\string\end{examplelist}}}

\newcommand{\l@example}{\@dottedtocline{1}{1.5em}{2.3em}}

\newcommand*{\htmlexampleref}[4]{\htmlref{#1}{#2}}
\newcommand*{\htmlexampleitem}{\item \htmlexampleref}

\newenvironment{examplelist}{\begin{enumerate}}{\end{enumerate}}

\newcounter{example}
%\begin{example}[html backref]{title}{label}
\newenvironment{example}[3][\@htmlex@backref]{%
  \def\@htmlex@backref{\@html@nodename}%
  \xminisec{\refstepcounter{example}\label{#3}Example \theexample. #2}%
  \def\@the@exampletitle{#2}%
  \def\@the@examplelabel{#3}%
  \@onelevel@sanitize\@the@exampletitle
  \immediate\write\tf@hloe{\string\htmlexampleitem{\@the@exampletitle}{#3}{#1}{\theexample}}%
  \addcontentsline{loe}{example}{\protect\numberline{\theexample}{\@the@exampletitle}}\nopagebreak
}%
{%
  \label{endex.\@the@examplelabel}%
  \vskip2pt plus 2pt minus 1pt\relax
  \par\noindent\mbox{}\hfill
  \separator
  \hfill\mbox{}\par
  \vskip5pt plus 6pt minus 2pt\relax
}

\newwrite\tf@hlox

\newcommand{\listofexercises}{%
  \chapter*{List of Exercises}
  \addcontentsline{toc}{chapter}{List of Exercises}%
  \markboth{List of Exercises}{List of Exercises}%
  \@starttoc{lox}
  \immediate\openout\tf@hlox \jobname.hlox\relax
  \immediate\write\tf@hlox{\string\begin{exerciselist}}%
}

\AtEndDocument{\immediate\write\tf@hlox{\string\end{exerciselist}}}

\newcommand{\l@exercise}{\@dottedtocline{1}{1.5em}{2.3em}}

\newcommand*{\htmlexerciseref}[4]{\htmlref{#1}{#2}}
\newcommand*{\htmlexerciseitem}{\item \htmlexerciseref}

\newenvironment{exerciselist}{\begin{enumerate}}{\end{enumerate}}

\newcounter{exercise}
%\begin{exercise}[html backref]{title}{label}
\newenvironment{exercise}[3][\@htmlex@backref]{%
  \def\@htmlex@backref{\@html@nodename}%
  \xminisec{\refstepcounter{exercise}\label{#3}Exercise \theexercise. #2}%
  \def\@the@exercisetitle{#2}%
  \def\@the@exerciselabel{#3}%
  \@onelevel@sanitize\@the@exercisetitle
  \immediate\write\tf@hlox{\string\htmlexerciseitem{\@the@exercisetitle}{#3}{#1}{\theexercise}}%
  \addcontentsline{lox}{exercise}{\protect\numberline{\theexercise}{\@the@exercisetitle}}\nopagebreak
}%
{%
  \label{endex.\@the@exerciselabel}%
  \vskip2pt plus 2pt minus 1pt\relax
  \par\noindent\mbox{}\hfill
  \separator
  \hfill\mbox{}\par
  \vskip5pt plus 6pt minus 2pt\relax
}

\newcommand{\faqlink}[2]{%
  \href{http://www.tex.ac.uk/cgi-bin/texfaq2html?label=#2}{#1}%
}

\newcommand{\faq}[2]{%
  \marginpar
  {%
    \parbox[t]{\marginparwidth}{%
      \raggedright
      \ifcase\adminmedia
        \small
      \or
        \footnotesize
      \or
       \footnotesize\em
      \fi
      [FAQ: \faqlink{#1}{#2}]%
    }\par
  }%
}

\newcommand{\ukenglishdate}{%
  \renewcommand{\fmtord}[1]{##1}%
  \renewcommand*{\formatdate}[3]{%
  \@day=##1\relax\@month=##2\relax\@year=##3\relax
  \ordinalnum{\the\@day}\ \monthname[\@month] \the\@year}%
}

\DeclareMathOperator{\card}{card}
\DeclareMathOperator{\diag}{diag}
\DeclareMathOperator*{\mode}{mode}

\newcommand*{\refname}{References}

\newcommand*{\backslashcircumsort}{%
  \string\~%
}

\newcommand*{\backslashtildesort}{%
  \string\~%
}

% option format
\providecommand*{\optfmt}[1]{\texttt{#1}}

\newcommand{\actualchar}{?}

% display and index class files:

\DeclareRobustCommand*{\icls}[1]{%
  \cls{#1}%
  \index{class files (.cls)\actualchar class files
(\texttt{.cls})!#1\actualchar\cls{#1}|hyperpage}%
  \index{#1 class\actualchar\cls{#1} class|hyperpage}%
}

\DeclareRobustCommand*{\genclsopt}[1]{%
 \optfmt{#1}%
 \index{class file options (general)!#1\actualchar\optfmt{#1}|hyperpage}%
}

\DeclareRobustCommand*{\clsopt}[2]{%
  \optfmt{#2}%
  \index{#2 (#1)\actualchar\texttt{#2} (\cls{#1})|hyperpage}%
  \index{#1 class\actualchar\cls{#1} class!#2\actualchar\texttt{#2}|hyperpage}%
}

\let\nxclsopt\genclsopt

% display option, index package and option
% \styopt{package}{option}
\DeclareRobustCommand*{\styopt}[2]{\optfmt{#2}%
  \index{#1 package\actualchar\sty{#1} 
    package!#2\actualchar\optfmt{#2}|hyperpage}%
  \index{#2 (#1)\actualchar\optfmt{#2} (\sty{#1})|hyperpage}%
}

% display and index bibtex style files
\DeclareRobustCommand*{\ibst}[1]{%
  \bst{#1}%
  \index{bibtex styles (.bst)\actualchar\bibtex\ styles
(\texttt{.bst})!#1\actualchar \bst{#1}|hyperpage}%
}

\DeclareRobustCommand*{\bst}[1]{\textsf{#1}}

% display and index package:
\DeclareRobustCommand*{\isty}[1]{%
 \sty{#1}%
 \index{#1 package\actualchar\sty{#1} package|hyperpage}%
}

\DeclareRobustCommand*{\istystart}[1]{\sty{#1}\index{#1
package\actualchar\sty{#1} package|(hyperpage}}
\DeclareRobustCommand*{\istyend}[1]{\sty{#1}\index{#1
package\actualchar\sty{#1} package|)hyperpage}}

% KOMA Script class option
\DeclareRobustCommand*{\scrclsopt}[2][]{%
 \ifstrempty{#1}%
 {%
   \optfmt{#2}%
 }%
 {%
   \optfmt{#2\discretionary{}{}{}=\discretionary{}{}{}#1}%
 }%
 \index{KOMA-Script class options!#2\actualchar\optfmt{#2}|hyperpage}%
 \index{#2 (KOMA)\actualchar\optfmt{#2} (KOMA)|hyperpage}%
}

\let\nxscrclsopt\scrclsopt

% tikz library
\newcommand*{\tikzlibrary}[1]{\texttt{#1}}
\DeclareRobustCommand*{\itikzlibrary}[1]{\tikzlibrary{#1}%
  \index{tikz library\actualchar\sty{tikz} 
   library!#1\actualchar\tikzlibrary{#1}|hyperpage}%
}

% Command key-val option
% \ikeyvalopt{csname}{key}
\DeclareRobustCommand*{\ikeyvalopt}[2]{\index{#1\actualchar\protect\cmdname{#1}!#2\actualchar\texttt{#2}|hyperpage}\texttt{#2}}
\DeclareRobustCommand*{\keyvalopt}[1]{\texttt{#1}}

\newcommand*{\iglskey}[1]{%
 \ikeyvalopt{newglossaryentry}{#1}%
}

\newcommand*{\iglslinkkey}[1]{%
 \ikeyvalopt{glslink}{#1}%
}

\newcommand*{\ipglskey}[1]{%
 \ikeyvalopt{printglossary}{#1}%
}

% display and index bib entry
\DeclareRobustCommand*{\ibibentry}[1]{%
  \bibentry{#1}%
  \index{bib entry types!#1\actualchar\bibentry{#1}|hyperpage}%
}

\DeclareRobustCommand*{\bibentry}[1]{\textsf{#1}}

% display and index bib field
\DeclareRobustCommand*{\ibibfield}[1]{%
  \bibfield{#1}%
  \index{bib entry fields!#1\actualchar\bibfield{#1}|hyperpage}%
}

\DeclareRobustCommand*{\bibfield}[1]{\textsf{#1}}

% index page style
\newcommand{\idxpagestyle}[1]{\index{page
style!#1\actualchar\texttt{#1}|hyperpage}}
% display and index page style
\DeclareRobustCommand*{\ipagestyle}[1]{\pagestylefmt{#1}\idxpagestyle{#1}}

\newcommand{\pagestylefmt}[1]{\texttt{#1}}

\newcommand{\pagenumberingfmt}[1]{\texttt{#1}}
% index page numbering
\newcommand{\indexpagenumbering}[1]{%
  \index{page numbering!#1\actualchar\pagenumberingfmt{#1}|hyperpage}%
}
% display page numbering
\newcommand{\ipagenumbering}[1]{%
  \pagenumberingfmt{#1}\indexpagenumbering{#1}%
}

% display and index unit of measurement
\newcommand*{\iunit}[2]{\texttt{#1}\index{#1 (#2)\actualchar\texttt{#1}
(#2)|hyperpage}\index{units!#1 (#2)\actualchar\texttt{#1} (#2)|hyperpage}}

\renewcommand{\see}[2]{\par\qquad\emph{\seename} #1}

\newcommand{\ibackslashcircumdef}{\cmdname{\textasciicircum}%
\index{\string\^\actualchar\protect\cmdname{\protect\textasciicircum}|indexdef}}

\newcommand{\ibackslashtildedef}{\cmdname{\textasciitilde}%
\index{\string\~\actualchar\protect\cmdname{\protect\textasciitilde}|indexdef}}

\newcommand*{\iquestiondowncmd}{\glsi{questiondown}}
\newcommand*{\iexclamdowncmd}{\glsi{exclamdown}}

\newcommand*{\idblquote}{\texttt{\string"}\index{""\actualchar\texttt{\string""}|hyperpage}}

\pgfarrowsdeclarecombine
  {combined}{combined}{triangle 60}{triangle 60}{|}{|}%

\newsavebox\tabbox
\sbox\tabbox
{%
  \scalebox{0.5}%
  {%
    \tikz
    {%
      \draw[combined-,thick] (0pt,0pt) -- (2em,0pt);
      \draw[-combined,thick] (0pt,-1.5ex) -- (2em,-1.5ex);
    }%
  }%
}

\newsavebox\enterbox
\sbox\enterbox
{%
  \scalebox{0.5}%
  {%
    \tikz
    {%
      \draw[triangle 60-,thick] (0pt,-.5ex) (0pt,0pt) -| (2em,1ex);
    }%
  }%
}

\newcommand*{\tabsym}{\raisebox{.5ex}{\keystroke{\space\usebox\tabbox\space}}}

\newcommand*{\itabsym}{\tabsym\index{tab character\actualchar tab character
\string\tabsym|hyperpage}}

\newcommand*{\entersym}{\raisebox{.5ex}{\keystroke{\space\usebox\enterbox\space}}}

\newcommand*{\boldc}{\ensuremath{\boldsymbol{c}}}

\newcommand*{\supdag}{\textsuperscript{\dag}}
\newcommand*{\supddag}{\textsuperscript{\ddag}}
\newcommand*{\supdagddag}{\textsuperscript{\dag\ddag}}

\edef\percentsym{\expandafter\@gobble\string\%}
\edef\hashsym{\expandafter\@gobble\string\#}
\edef\dbhashsym{\hashsym\hashsym}
\edef\ampsym{\expandafter\@gobble\string\&}
\edef\dollarsym{\expandafter\@gobble\string\$}
\edef\underscoresym{\expandafter\@gobble\string\_}
\edef\circumsym{\expandafter\@gobble\string\^}
\edef\tildesym{\expandafter\@gobble\string\~}
\edef\leftbracesym{\expandafter\@gobble\string\{}
\edef\rightbracesym{\expandafter\@gobble\string\}}
\edef\dbbackslashsym{\string\\}
\edef\backslashsym{\expandafter\@gobble\dbbackslashsym}

% protected makeindex special characters
\newcommand*{\pleftbracesym}{"\leftbracesym}
\newcommand*{\prightbracesym}{"\rightbracesym}

\DeclareRobustCommand*{\quotecs}{\cmdname{"}}
\DeclareRobustCommand*{\umlautaccent}[1]{\"{#1}}

\DeclareRobustCommand*{\doublebar}[1]{\ensuremath{\|}}

\newcommand*{\mathdoubleprime}{\ensuremath{''}}
\newcommand*{\mathsingleprime}{\ensuremath{'}}
\newcommand*{\mathtilde}{\ensuremath{\sim}}
\newcommand*{\mathvee}{\ensuremath{\vee}}
\newcommand*{\mathwedge}{\ensuremath{\wedge}}
\newcommand*{\mathequiv}{\ensuremath{\equiv}}
\newcommand*{\mathsum}{\ensuremath{\sum}}
\newcommand*{\mathlvert}{\ensuremath{\lvert}}
\newcommand*{\mathrvert}{\ensuremath{\rvert}}
\newcommand*{\mathepsilon}{\ensuremath{\epsilon}}
\newcommand*{\mathleftarrow}{\ensuremath{\leftarrow}}
\newcommand*{\mathcalS}{\ensuremath{\mathcal{S}}}

\newcommand{\exclamsym}{!}
\newcommand{\questionsym}{?}
\newcommand{\atsym}{@}
\newcommand{\xatsym}{"@}
\newcommand{\vbarsym}{|}
\newcommand{\doublequotesym}{"}
\newcommand{\slashsym}{/}
\newcommand{\emDashcs}{-{}-{}-}
\newcommand{\enDashcs}{-{}-}
\newcommand{\dashcs}{-}
\newcommand{\pluscs}{+}
\newcommand{\mprime}{$'$}
\newcommand{\mdprime}{$''$}

%\newcommand{\quoteleftcs}{`}
%\newcommand{\quotedblleftcs}{`{}`}
%\newcommand{\quoterightcs}{'}
%\newcommand{\quotedblrightcs}{'{}'}
\newcommand{\quoteleftcs}{\textasciigrave}
\newcommand{\quotedblleftcs}{\textasciigrave\textasciigrave}
\newcommand{\quoterightcs}{\textquotesingle}
\newcommand{\quotedblrightcs}{\textquotesingle\textquotesingle}
\newcommand{\questiondowncs}{?\textasciigrave}
\newcommand{\exclamdowncs}{!\textasciigrave}
\newcommand{\opensqsym}{[}
\newcommand{\closesqsym}{]}
\newcommand{\openparensym}{(}
\newcommand{\closeparensym}{)}
\newcommand{\commasym}{,}
\newcommand{\colonsym}{:}
\newcommand{\semicolonsym}{;}
\newcommand{\lesssym}{\string<}
\newcommand{\greatersym}{\string>}
\newcommand{\periodsym}{.}
\newcommand{\equalsym}{=}

\newcommand*{\dash}{\,---\,}
\providecommand*{\cdash}{\dash}
\DeclareRobustCommand{\pLaTeX}{\LaTeX}
\DeclareRobustCommand{\pTeX}{\TeX}

\let\hidden\@gobble

\newcommand{\minus}{\ensuremath{-}}

\newcommand{\doublequestionmark}{%
  ??\index{"?"? (undefined reference)}%
}

\DeclareRobustCommand{\symbolcmddef}[1]{%
 \cmdname{#1}%
 \indexsymbolcmddef{#1}%
}

\DeclareRobustCommand{\indexsymbolcmddef}[1]{%
 \index{\backslashsym#1\actualchar\protect\cmdname{#1}|indexdef}%
}

\DeclareRobustCommand{\indexspsymbolcmddef}[1]{%
{\edef\sortkey{\expandafter\string\csname#1\endcsname}%
\index{\sortkey \actualchar\protect\cmdname{"#1}|indexdef}}}

\DeclareRobustCommand{\indexspsymbolcmd}[1]{%
{\edef\sortkey{\expandafter\string\csname#1\endcsname}%
\index{\sortkey \actualchar\protect\cmdname{"#1}|hyperpage}}}

\DeclareRobustCommand{\symbolcmd}[1]{%
 \cmdname{#1}%
 \indexsymbolcmd{#1}%
}

\DeclareRobustCommand{\indexsymbolcmd}[1]{%
  \index{\backslashsym#1\actualchar\protect\cmdname{#1}|hyperpage}%
}

% index a command definition, but don't display it in the text
\newcommand{\indexComdef}[1]{\index{#1\actualchar\protect\cmdname{#1}|indexdef}}

% index a command name, but don't display it in the text
\newcommand{\indexCom}[1]{\index{#1\actualchar\protect\cmdname{#1}|hyperpage}}

\newcommand*{\iatsym}{\texttt{@}\index{"@\actualchar\texttt {\string\atsym}|hyperpage}}

\DeclareRobustCommand{\indexspsymbol}[1]{%
{\edef\sortkey{\expandafter\string\csname#1\endcsname}%
\index{\sortkey \actualchar\protect\texttt{"#1}|hyperpage}}}

\newcommand*{\keywordfmt}[1]{\emph{#1}}

\newcommand*{\keyword}[2][\keywordtext]{%
 \def\keywordtext{#2}%
 \keywordfmt{#2}\index{#1|hyperpage}%
}

\renewcommand{\glossarysection}[2][\@gls@title]{%
  \def\@gls@title{#2}%
  \protected@edef\do@def@glo@section{\noexpand\chapter[#1]{#2}}%
  \do@def@glo@section
}

\newcommand{\enter}{\scalebox{0.8}{\input{pictures/entersymbol}}}

\DeclareRobustCommand{\cmdname}[1]{\texttt{\char`\\#1}}
\DeclareRobustCommand{\parg}[1]{%
  \discretionary{}{}{}\texttt(#1\texttt)}
\newcommand{\pargm}[1]{\parg{\meta{#1}}}

\DeclareRobustCommand{\meta}[1]{\textnormal{\ensuremath{\langle}\emph{#1}\ensuremath{\rangle}}}

\DeclareRobustCommand{\dmeta}[1]{\discretionary{}{}{}\meta{#1}\discretionary{}{}{}}

\DeclareRobustCommand{\marg}{%
  \@ifstar\@smarg\@marg
}

\newcommand{\@marg}[1]{%
  \discretionary{}{}{}\glsni{leftbracechar}#1\glsni{rightbracechar}%
}

\newcommand{\@smarg}[1]{%
  \discretionary{}{}{}\texttt{\leftbracesym}#1\texttt{\rightbracesym}%
}

\DeclareRobustCommand{\margm}{%
  \@ifstar\@smargm\@margm
}

\newcommand{\@margm}[1]{%
  \discretionary{}{}{}\glsni{leftbracechar}\meta{#1}\glsni{rightbracechar}%
}

\newcommand{\@smargm}[1]{%
  \discretionary{}{}{}\texttt{\leftbracesym}\meta{#1}\texttt{\rightbracesym}%
}

\DeclareRobustCommand{\oarg}{%
  \@ifstar\@soarg\@oarg
}

\DeclareRobustCommand{\@oarg}[1]{%
  \discretionary{}{}{}\glsni{opt.opensq}#1\glsni{opt.closesq}}

\newcommand{\@soarg}[1]{%
  \discretionary{}{}{}\texttt[#1\texttt]}

\DeclareRobustCommand{\oargm}{%
  \@ifstar\@soargm\@oargm
}

\newcommand{\@oargm}[1]{%
  \discretionary{}{}{}\glsni{opt.opensq}\meta{#1}\glsni{opt.closesq}}

\newcommand{\@soargm}[1]{%
  \discretionary{}{}{}\texttt[\meta{#1}\texttt]}

\newcommand{\comma}{,\discretionary{}{}{}}
\newcommand{\hyphen}{-\penalty\exhyphenpenalty}

\newcommand{\dequals}{\discretionary{}{}{}=\discretionary{}{}{}}
\newcommand{\dcolon}{\discretionary{}{}{}:\discretionary{}{}{}}

\DeclareRobustCommand{\envname}[1]{\textsf{#1}}

\DeclareRobustCommand{\cls}[1]{\textsf{#1}}
\DeclareRobustCommand{\sty}[1]{\textsf{#1}}
\newcommand{\counter}[1]{\textsf{#1}}

\DeclareRobustCommand{\appname}[1]{\texttt{#1}}
\newcommand{\iappname}[2][\appnamesort]{%
{\def\appnamesort{#2}%
\appname{#2}%
\index{#1\actualchar\appname{#2}|hyperpage}}}

\newcommand*{\appopt}{\@ifstar\s@appopt\@appopt}
\newcommand*{\iappopt}{\@ifstar\s@iappopt\@iappopt}

\newcommand*{\@appopt}[1]{\texttt{-{}-#1}}
\newcommand*{\@iappopt}[2]{\@appopt{#2}%
  \index{#1\actualchar\string\appname{#1}!#2\actualchar\@appopt{#2}|hyperpage}}

\newcommand*{\s@appopt}[1]{\texttt{-#1}}
\newcommand*{\s@iappopt}[2]{\s@appopt{#2}%
  \index{#1\actualchar\string\appname{#1}!#2\actualchar\s@appopt{#2}|hyperpage}}

\DeclareRobustCommand{\directive}[1]{\texttt{#1}}
\newcommand{\idirective}[1]{%
\directive{#1}%
\index{arara directives\actualchar\string\appname{arara} directives!#1\actualchar\string\directive{#1}|hyperpage}}

\newcommand{\texdistro}[1]{\texttt{#1}}
\newcommand{\itexdistro}[2][\distrosort]{%
{%
  \def\distrosort{#2}%
  \texdistro{#2}%
  \index{#1\actualchar\texdistro{#2}|hyperpage}%
  \index{TeX Distributions\actualchar\protect\TeX\
Distributions!#1\actualchar\protect\texdistro{#2}|hyperpage}%
}%
}

\newcommand{\perldistro}[1]{\texttt{#1}}

\newcommand{\menuto}{\discretionary{}{}{}$\rightarrow$\discretionary{}{}{}}

\newcommand{\startmenu}[1]{\begin{center}\sffamily Start$\rightarrow$ Programs $\rightarrow$ #1\end{center}}

\newcommand{\menu}[1]{\textsf{#1}}

\DeclareRobustCommand{\BiBTeX}{\textsc{Bib}\TeX}
\newcommand*{\PDFLaTeX}{PDF\LaTeX}
\newcommand*{\PDFTeX}{PDF\TeX}
\newcommand*{\ConTeXt}{Con\TeX{}t}

\newcommand*{\mname}{\meta{name}}

\newcommand{\dq}[1]{``#1''}
\newcommand{\sq}[1]{`#1'}
\newcommand{\upsq}[1]{\textquotesingle #1\textquotesingle}
\newcommand{\updq}[1]{\string"#1\string"}

\renewcommand{\cleardoublepage}{%
 \clearpage
 \if@twoside
  \ifodd\c@page
  \else
   \hbox{}%
   \thispagestyle{empty}%
   \clearpage
  \fi
 \fi
}

\newcommand*{\supregistered}{\textsuperscript{\textregistered}}

\newcommand{\footnotesymbols}{%
\@fnsymbol1 \@fnsymbol2 \@fnsymbol3 \@fnsymbol4 \@fnsymbol5
\@fnsymbol6 \@fnsymbol7 \@fnsymbol8 \@fnsymbol9}

\newlength\imgHeight
\newlength\maxImageHeight

\setlength{\maxImageHeight}{0.8\textheight}

\newlength\imgWidth
\newlength\maxImageWidth
\setlength{\maxImageWidth}{\textwidth}

\define@key{Gin}{alt}{}
\define@boolkey{Gin}{result}[true]{}

\newcommand{\imgresultfmt}[1]{%
 \leftblocklabel{\usebox\Output}{}%
 \hfill\vtop{\vskip0pt\hbox{#1}}%
 \rightblocklabel{\usebox\Output}{}\par
}

\newcommand{\incGraphics}[2][]{%
 \begingroup
  \setkeys{Gin}{#1}%
  \ifKV@Gin@result
   \let\@incimgresult\imgresultfmt
  \else
   \let\@incimgresult\@firstofone
  \fi
  \def\admin@imgfile{#2}%
  \ifnum\adminmedia=\ADMINscreen\relax
   \IfFileExists{#2-screen.pdf}%
    {\def\admin@imgfile{#2-screen}}%
    {\IfFileExists{#2-screen.png}
     {\def\admin@imgfile{#2-screen}}{}}%
  \fi
  \setlength{\maxImageWidth}{\linewidth}%
  \settoheight{\imgHeight}{\includegraphics[#1]{\admin@imgfile}}%
  \ifdim\imgHeight>\maxImageHeight
    \settowidth{\imgWidth}{\includegraphics[#1,height=\maxImageHeight]{\admin@imgfile}}%
    \ifdim\imgWidth>\maxImageWidth
      \@incimgresult{\includegraphics[#1,width=\maxImageWidth]{\admin@imgfile}}%
    \else
      \@incimgresult{\includegraphics[#1,height=\maxImageHeight]{\admin@imgfile}}%
    \fi
  \else
    \settowidth{\imgWidth}{\includegraphics[#1]{\admin@imgfile}}%
    \ifdim\imgWidth>\maxImageWidth
      \@incimgresult{\includegraphics[#1,width=\maxImageWidth]{\admin@imgfile}}%
    \else
     \@incimgresult{\includegraphics[#1]{\admin@imgfile}}%
    \fi
  \fi
 \endgroup
}

\newsavebox{\boxcontents}

\newcommand*{\includeteximage}[2][]{%
  \setlength{\maxImageWidth}{\linewidth}%
  \def\Gin@req@sizes{%
    \Gin@req@height\Gin@nat@height
    \Gin@req@width\Gin@nat@width}%
  \begingroup
    \@tempswafalse
    \let\input@path\Ginput@path
    \toks@{\InputIfFileExists{#2}{}{\@warning{File `#1' not found}}}%
    \setkeys{Gin}{#1}%
    \ifKV@Gin@result
     \let\@incimgresult\imgresultfmt
    \else
     \let\@incimgresult\@firstofone
    \fi
    \Gin@esetsize
    \sbox\boxcontents{\the\toks@}%
    \settoheight{\imgHeight}{\usebox\boxcontents}%
    \settodepth{\@ff@tmp@y}{\usebox\boxcontents}%
    \addtolength{\imgHeight}{\@ff@tmp@y}%
    \ifdim\imgHeight>\maxImageHeight
      \ifdim\maxImageHeight<\maxImageWidth\relax
        \@incimgresult{\resizebox{!}{\maxImageHeight}{\usebox\boxcontents}}%
      \else
        \settowidth{\imgWidth}{\usebox\boxcontents}%
        \ifdim\imgWidth>\maxImageWidth
          \@incimgresult{\resizebox{\maxImageWidth}{!}{\usebox\boxcontents}}%
        \else
          \@incimgresult{\resizebox{!}{\maxImageHeight}{\usebox\boxcontents}}%
        \fi
      \fi
    \else
      \settowidth{\imgWidth}{\usebox\boxcontents}%
      \ifdim\imgWidth>\maxImageWidth
       \@incimgresult{\resizebox{\maxImageWidth}{!}{\usebox\boxcontents}}%
      \else
        \@incimgresult{\usebox\boxcontents}%
      \fi
    \fi
  \endgroup
}

\newcommand*{\incPgfOrGraphics}[2][]{%
  \begingroup
    \let\input@path\Ginput@path
    \IfFileExists{#2.tex}{\includeteximage[#1]{#2}}{\incGraphics[#1]{#2}}%
  \endgroup
}

\newcommand*{\incFboxPgfOrGraphics}[2][]{%
 \frame{\incPgfOrGraphics[#1]{#2}}%
}

\newcommand{\indexdef}[1]{\textit{\hyperpage{#1}}}

% footnotes for minipage examples
\newcommand{\mpexfootnote}[1]{%
\renewcommand{\thempfootnote}{\textcolor{black}{\alph{mpfootnote}}}%
\footnote{#1}}

\newcommand{\labelledbox}[2]{\begin{tabular}{r}\doublebox{#1}\\\scriptsize\textsf{#2}\end{tabular}}
\newcommand{\labelled}[2]{\begin{tabular}{r}#1\\\scriptsize\textsf{#2}\end{tabular}}

\ifnum\adminmedia=\ADMINbook
  \definecolor{inputcol}{gray}{0.1}
  \definecolor{outputcol}{gray}{0.1}
\else
  \definecolor{inputcol}{cmyk}{1,0,0,0}
  \definecolor{outputcol}{rgb}{0.6,0,1}
\fi

\newcommand{\inputlabelformat}[1]{\textcolor{inputcol}{\scriptsize\sffamily #1}}
\newcommand{\outputlabelformat}[1]{\textcolor{outputcol}{\scriptsize\sffamily #1}}

\newcommand{\toparrow}{\ensuremath{\overline\uparrow}}
\newcommand{\botarrow}{\ensuremath{\underline\downarrow}}

\newlength\minremainspace

\ifnum\adminmedia=\ADMINbook

  \newcommand*{\leftblocklabel}[2]{%
    \ifthispageodd{}{\llap{\makebox[\marginparwidth][r]{#2 #1}\hspace{\marginparsep}}}%
  }

  \newcommand*{\rightblocklabel}[2]{%
    \hfill\ifthispageodd{\rlap{\hspace{\marginparsep}\makebox[\marginparwidth][l]{#1 #2}}}{}%
  }

  \newcommand*{\marginblocklabel}[2]{%
    \ifthispageodd
    {%
      \makebox[\marginparwidth][l]{#1 #2}%
    }%
    {%
      \makebox[\marginparwidth][r]{#2 #1}%
    }%
  }

 \newcommand*{\marginlabel}[1]{%
    \ifthispageodd
    {%
      \makebox[\marginparwidth][l]{#1}%
    }%
    {%
      \makebox[\marginparwidth][r]{#1}%
    }%
 }

 \setlength\minremainspace{50pt}

\else
  \newcommand*{\leftblocklabel}[2]{}
  \newcommand*{\rightblocklabel}[2]{%
    \hfill\rlap{\hspace{\marginparsep}\makebox[\marginparwidth][l]{#1 #2}}%
  }

 \newcommand*{\marginblocklabel}[2]{#1 #2}
 \newcommand*{\marginlabel}[1]{#1}
 \setlength\minremainspace{42pt}
\fi

% Changing the colour in the margin occasionally causes problems
% so store the coloured marginal text in save boxes.

\newsavebox\Input
\newsavebox\Output
\newsavebox\InputT
\newsavebox\OutputT
\newsavebox\CodeT
\newsavebox\InputB
\newsavebox\OutputB
\newsavebox\CodeB
\newsavebox\Code
\newsavebox\Shell
\newsavebox\SQL

\sbox\Input{\inputlabelformat{Input}}
\sbox\Output{\outputlabelformat{Output}}
\sbox\Code{\inputlabelformat{Code}}
\sbox\Shell{\inputlabelformat{Shell}}
\sbox\SQL{\inputlabelformat{SQL}}

\sbox\InputT{\inputlabelformat{%
   \toparrow
}}

\sbox\OutputT{\outputlabelformat{%
  \toparrow
 }}

\sbox\CodeT{\inputlabelformat{%
  \toparrow
}}

\sbox\InputB{\inputlabelformat{%
  \botarrow
}}

\sbox\OutputB{\outputlabelformat{%
  \botarrow
}}

\sbox\CodeB{\inputlabelformat{%
  \botarrow
}}

\newcommand{\inputruleB}{%
 \vspace*{-5pt}\par\penalty10000\noindent
 \raisebox{4pt}{%
  \leftblocklabel{\usebox\InputB}{\usebox\Input}%
 }%
 \textcolor{inputcol}{\rule{0.4pt}{5pt}%
 \leaders\hrule\hfill\rule{0.4pt}{5pt}}%
 \raisebox{4pt}{%
  \rightblocklabel{\usebox\InputB}{\usebox\Input}%
 }%
 \par\penalty-\@lowpenalty}

\newcommand{\outputruleB}{\par\penalty10000\noindent
 \raisebox{4pt}{%
  \leftblocklabel{\usebox\OutputB}{\usebox\Output}%
 }%
 \textcolor{outputcol}{\rule{0.4pt}{5pt}%
 \leaders\hrule\hfill\rule{0.4pt}{5pt}}%
 \raisebox{4pt}{%
  \rightblocklabel{\usebox\OutputB}{\usebox\Output}%
 }%
 \par\penalty-\@lowpenalty
}

\newcommand{\coderuleB}{\par\penalty10000\noindent
 \raisebox{4pt}{%
  \leftblocklabel{\usebox\CodeB}{\usebox\Code}%
 }%
 \textcolor{inputcol}{\rule{0.4pt}{5pt}%
 \leaders\hrule\hfill\rule{0.4pt}{5pt}}%
 \raisebox{4pt}{%
  \rightblocklabel{\usebox\CodeB}{\usebox\Code}%
 }%
 \par\penalty-\@lowpenalty}

\newcommand{\inputruleT}{\par\penalty-\@lowpenalty\noindent
 \raisebox{-8pt}{%
  \leftblocklabel{\usebox\InputT}{\usebox\Input}%
 }%
 \textcolor{inputcol}{\rule[-5pt]{0.4pt}{5pt}%
 \leaders\hrule\hfill\rule[-5pt]{0.4pt}{5pt}}%
 \raisebox{-8pt}{%
  \rightblocklabel{\usebox\InputT}{\usebox\Input}%
 }%
 \par\penalty10000}

\newcommand{\outputruleT}{\par\penalty-\@lowpenalty\noindent
 \raisebox{-8pt}{%
  \leftblocklabel{\usebox\OutputT}{\usebox\Output}%
 }%
 \textcolor{outputcol}{\rule[-5pt]{0.4pt}{5pt}%
 \leaders\hrule\hfill\rule[-5pt]{0.4pt}{5pt}}%
 \raisebox{-8pt}{%
  \rightblocklabel{\usebox\OutputT}{\usebox\Output}%
 }%
 \par\penalty10000}

\newcommand{\coderuleT}{\par\penalty-\@lowpenalty\noindent
 \raisebox{-8pt}{%
  \leftblocklabel{\usebox\CodeT}{\usebox\Code}%
 }%
 \textcolor{inputcol}{\rule[-5pt]{0.4pt}{5pt}%
 \leaders\hrule\hfill\rule[-5pt]{0.4pt}{5pt}}%
 \raisebox{-8pt}{%
  \rightblocklabel{\usebox\CodeT}{\usebox\Code}%
 }%
 \par\penalty10000}

\newcommand{\labelledinputruleT}[1]{%
  \par\penalty-\@lowpenalty\noindent
 \textcolor{inputcol}%
 {%
    \rule[-5pt]{0.4pt}{5pt}\rule{1em}{.4pt}%
    \fbox{#1}\leaders\hrule\hfill\rule[-5pt]{0.4pt}{5pt}%
 }%
 \par\penalty10000\relax
}

\newcommand*{\nonodesubsection}{\subsection*}

\newcommand*{\xminisec}[1]{%
 \@check@remainingspace[3.5\baselineskip]%
 \minisec{#1}%
}

\newcommand*{\@check@remainingspace}[1][\minremainspace]{%
  \penalty-\@medpenalty\par
  \mbox{}%
  \@tempdimc\pagegoal
  \advance\@tempdimc-\pagetotal
  \noindent
  \setlength{\@tempdimb}{#1}%
  \ifdim\@tempdimb>\@tempdimc\relax
   \pagebreak[2]%
  \fi
  \vspace{-\baselineskip}%
  \vspace{-\parskip}%
}

\newcommand*{\blockskip}{\vskip1.5ex plus .2ex minus .2ex}

% define commands governing how input code and 
% corresponding output should be displayed

\newenvironment{bcode}%
 {%
   \@check@remainingspace
   \blockskip
   \coderuleT
   \ttfamily\frenchspacing\obeyspaces\par\raggedright\noindent\ignorespaces
 }%
 {%
   \par\coderuleB
   \par\penalty-\@lowpenalty
   \blockskip
   \noindent\ignorespacesafterend
 }

%Single line of input
\newenvironment{codeS}[1][\linewidth]%
{%
  \@check@remainingspace
  \blockskip
  \par\noindent
  \leftblocklabel{\usebox\Input}{}%
  \begin{ttfamily}\raggedright\frenchspacing
}%
{%
  \end{ttfamily}%
  \rightblocklabel{\usebox\Input}{}%
  \par\penalty-\@lowpenalty
  \blockskip
  \noindent\ignorespacesafterend
}

%Multiple lines of input
\newenvironment{code}[1][\linewidth]%
{%
  \@check@remainingspace
  \blockskip
  \inputruleT
  \begin{alltt}\penalty10000\vskip-1ex\relax
}%
{%
  \penalty10000\end{alltt}\penalty10000\vskip-1ex\relax\penalty10000
  \inputruleB
  \par\penalty-\@highpenalty
  \blockskip
  \noindent\ignorespacesafterend
}

\newsavebox\shellbox

%Shell command
\newenvironment{shell}[1][\linewidth]%
{%
  \let\shellnewline\newline
  \@check@remainingspace
  \blockskip
  \noindent
  \setlength{\dimen@}{#1}%
  \addtolength{\dimen@}{-2\fboxsep}%
  \addtolength{\dimen@}{-7.5\fboxrule}%
  \addtolength{\dimen@}{-1pt}%
  \begin{lrbox}{\shellbox}%
  \begin{minipage}{\dimen@}\ttfamily\raggedright\frenchspacing
}%
{%
 \par\end{minipage}%
 \end{lrbox}%
 \leftblocklabel{\usebox\Shell}{}%
 \doublebox{\usebox\shellbox}%
 \rightblocklabel{\usebox\Shell}{}%
 \par
 \penalty-\@lowpenalty
 \blockskip
 \noindent\ignorespacesafterend
}

%SQL command
\newenvironment{SQLshell}[1][\linewidth]%
{%
  \@check@remainingspace
  \blockskip
  \noindent
  \setlength{\dimen@}{#1}%
  \addtolength{\dimen@}{-2\fboxsep}%
  \addtolength{\dimen@}{-7.5\fboxrule}%
  \addtolength{\dimen@}{-1pt}%
  \begin{lrbox}{\shellbox}%
  \begin{minipage}{\dimen@}\ttfamily\raggedright\frenchspacing
}%
{%
 \par\end{minipage}%
 \end{lrbox}%
 \leftblocklabel{\usebox\SQL}{}%
 \doublebox{\usebox\shellbox}%
 \rightblocklabel{\usebox\SQL}{}%
 \par
 \penalty-\@lowpenalty
 \blockskip
 \noindent\ignorespacesafterend
}

\let\admin@org@chaptermark\chaptermark
\let\admin@org@sectionmark\sectionmark
\let\admin@org@subsectionmark\subsectionmark

\newcommand{\currenttitle}{}

\renewcommand{\chaptermark}[1]{%
 \markboth
 {\jdrheadingcase{%
   \ifnum \c@secnumdepth >\m@ne
   \if@mainmatter
   \@chapapp\ \thechapter. \
   \fi
   \fi #1%
 }}%
 {\jdrheadingcase{%
   \ifnum \c@secnumdepth >\m@ne
   \if@mainmatter
   \@chapapp\ \thechapter. \
   \fi
   \fi #1%
 }}%
 \renewcommand*{\currenttitle}{#1}%
}

\renewcommand{\sectionmark}[1]{%
 \markright
 {%
   \MakeMarkcase{\ifnumbered{section}{\sectionmarkformat}{}#1}%
 }%
 \renewcommand*{\currenttitle}{#1}%
}

\renewcommand{\subsectionmark}[1]{
 \admin@org@subsectionmark{#1}%
 \renewcommand*{\currenttitle}{#1}%
}

\newcommand{\l@listing}{%
  \bprot@dottedtocline{1}{1.5em}{3em}%
}

\newcounter{codelisting}
\newenvironment{codelisting}[2][]%
{%
  \@check@remainingspace
  \blockskip
  \refstepcounter{codelisting}%
  \def\codelisting@arg{#1}%
  \ifdefempty{\codelisting@arg}{}{\def\codelisting@arg{ (#1)}}%
%  \addcontentsline{loe}{listing}{%
%    \protect\numberline{\thecodelisting}\currenttitle\codelisting@arg}%
  \labelledinputruleT{\color{black}\htmladdnormallink{Listing
\thecodelisting\codelisting@arg}{\baseurl/latex/admin/html/examples/#2}\doifbook{\footnotemark}}%
  \marginpar{\raisebox{-8pt}{%
    \marginblocklabel{\usebox\InputT}{\usebox\Input}}}%
  \doifbook{\footnotetext{\url{\baseurl/latex/admin/html/examples/#2}}}%
  \nopagebreak
  \begin{ttfamily}\nopagebreak\par\noindent\raggedright\frenchspacing\ignorespaces
}%
{%
  \par\end{ttfamily}\nopagebreak
  \inputruleB
  \marginpar{\raisebox{4pt}{%
    \marginblocklabel{\usebox\InputB}{\usebox\Input}}}%
  \par
  \blockskip
  \noindent\ignorespacesafterend
}

\newcommand*{\examplecode}[1]{%
 \htmladdnormallinkfoot{\texttt{#1}}{\baseurl/latex/admin/html/examples/#1}}

\newcommand*{\ltxresultwidth}{\linewidth}

%Single line of output
\newenvironment{ltxresultS}[1][]%
{%
  \@check@remainingspace
  \blockskip
  \noindent
  \leftblocklabel{\usebox\Output}{}%
  \begin{minipage}{\ltxresultwidth}%
  \let\do\noexpand
}%
{%
  \end{minipage}%
  \rightblocklabel{\usebox\Output}{}%
  \blockskip
  \noindent\ignorespacesafterend
}

%Multiple lines of output
\newenvironment{ltxresultM}[1][]%
{%
  \@check@remainingspace
  \blockskip
  \outputruleT
  \let\do\noexpand
}%
{%
  \outputruleB
  \par
  \blockskip
  \noindent\ignorespacesafterend
}

\newenvironment{htmlresult}[1]{\ignorespaces}{\ignorespacesafterend}
\let\htmlresultimg\htmlresult
\let\endhtmlresultimg\endhtmlresult

\newcommand*{\definitionlabel}{\scriptsize\sffamily Definition}

\newenvironment{definition}[1][\linewidth]%
{%
  \@check@remainingspace
  \blockskip
  \noindent
  \begin{lrbox}{\boxcontents}
  \begin{minipage}{#1}\ttfamily\raggedright\frenchspacing
}%
{%
  \par
  \end{minipage}\end{lrbox}%
  \leftblocklabel{\definitionlabel}{}%
  \usebox{\boxcontents}%
  \hfill\rightblocklabel{\definitionlabel}{}%
  \par\penalty-\@lowpenalty
  \blockskip
  \noindent\ignorespacesafterend
}

\newenvironment{display}%
{%
  \par
  \global\hangindent=0pt\relax
  \vskip0.5\baselineskip
  \parindent=0pt\relax
  \obeylines
  \ttfamily
  \ignorespaces
}%
{%
  \par
  \vskip0.5\baselineskip
  \par\noindent\ignorespacesafterend
}

\DeclareRobustCommand{\csentryargitem}[1]{\item{\meta{#1}}}

\DeclareRobustCommand{\BeginArgList}{\begin{description}}
\DeclareRobustCommand{\EndArgList}{\end{description}}

\protected@write\@auxout{}{%
\string\html{\string\gdef\string\newlabelxx
\string#1\string#2\string#3\string#4\string#5\string#6{%
\string\oldnewlabel{\string#1}{{\string#2}{\string#3}}%
\string\sectionlabel{\string#1}{\string#4}}}}

\ifnum\adminmedia=\ADMINbook
  \DeclareRobustCommand*{\appendixref}[2][]{Appendix~\ref{#2}
    ({\suppresslevels\nameref{#2}})}
  \DeclareRobustCommand*{\chapterref}[2][]{Chapter~\ref{#2}
    ({\suppresslevels\nameref{#2}})}
  \DeclareRobustCommand*{\sectionref}[2][]{Section~\ref*{#2}}
\else
  \DeclareRobustCommand*{\appendixref}[2][]{\htmlref{\S\ref*{#2} {\suppresslevels\nameref*{#2}}}{#2}}
  \DeclareRobustCommand*{\chapterref}[2][]{\htmlref{\S\ref*{#2} {\suppresslevels\nameref*{#2}}}{#2}}
  \DeclareRobustCommand*{\sectionref}[2][]{\htmlref{\S\ref*{#2}}{#2}}
\fi

\renewcommand*{\thefootnote}{\arabic{footnote}}
\renewcommand*{\theHfootnote}{\theHchapter.\arabic{footnote}}

\let\@mkboth\markboth

\newsavebox\tmp@sbox

\newcommand{\warning}{%
  \par
  \sbox{\tmp@sbox}%
  {%
    \LARGE
    \fontencoding {U}%
    \fontfamily{futs}\selectfont
      \char 66\relax
  }%
  \settowidth\dimen@{\usebox\tmp@sbox\space}%
  \hangindent=\dimen@\relax
  \hangafter=-2\relax
  \noindent\makebox[0pt][l]{\hspace*{-\dimen@}\smash{\raisebox{-0.5\baselineskip}{\usebox\tmp@sbox}}}%
}
\newcommand*{\wrong}{\marginpar{\marginlabel{\LARGE\color{red}\ding{56}}}}
\newcommand*{\correct}{\marginpar{\marginlabel{\LARGE\color{green}\ding{52}}}}

% Difficulty level

\newcommand*{\basiclevelimg}{\includegraphics{level1}}
\newcommand*{\mediumlevelimg}{\includegraphics{level2}}
\newcommand*{\hardlevelimg}{\includegraphics{level3}}

\newcommand*{\levelsanchor}{\raisebox{1ex}{\hypertarget{levels}{}}}

\newrobustcmd*{\basiclevel}{%
 \hyperlink{levels}{\basiclevelimg}\space
}
\newrobustcmd*{\mediumlevel}{%
 \hyperlink{levels}{\mediumlevelimg}\space
}
\newrobustcmd*{\hardlevel}{%
 \hyperlink{levels}{\hardlevelimg}\space
}

\renewcommand*{\@starttoc}[1]{%
 \begingroup
 \def\basiclevel{\basiclevelimg\space}%
 \def\mediumlevel{\mediumlevelimg\space}%
 \def\hardlevel{\hardlevelimg\space}%
 \makeatletter
 \@input{\jobname.#1}%
 \if@filesw
  \expandafter\newwrite\csname tf@#1\endcsname
  \immediate\openout\csname tf@#1\endcsname\jobname .#1\relax
 \fi
 \@nobreakfalse
 \endgroup
}

% Computer Modern maths fonts
\DeclareMathAlphabet{\cmmathit}{T1}{cmr}{m}{it}
\DeclareMathAlphabet{\cmmathtt}{T1}{cmtt}{m}{n}
\DeclareMathAlphabet{\cmmathsf}{T1}{cmss}{m}{n}
\DeclareMathAlphabet{\cmmathbf}{T1}{cmr}{bx}{n}
\DeclareSymbolFontAlphabet{\cmmathcal}{symbols}
\DeclareSymbolFontAlphabet{\cmmathnormal}{letters}
\DeclareSymbolFontAlphabet{\cmmathrm}{operators}

\newcommand*{\cmr}[1]{{\fontfamily{cmr}\selectfont #1}}

\DeclareRobustCommand*{\indexglsindex}[1]{%
  \index{\glsindex{#1}|hyperpage}%
}

% \newacr{label}{short}{long}{url}

\newcommand*{\newacr}[4]{%
  \newglossaryentry{#1}{%
    type=acronym,
    sort={#2},
    name={#2\gloindex{#2}},
    text={#2\protect\index{#2|hyperpage}},
    first={#3\protect\index{#2|hyperpage}
(#2)},
    description={\makefirstuc{#3}\latex{~\cite{#1}}. \protect\url{#4}},%
    user4={#2}%
  }%
}

\let\newabbr\newacr

% \newacrnourl[ext desc][key val options]{label}{short}{long}
\newcommand*{\newacrnourl}[1][]{%
  \def\@acro@extdesc{#1}%
  \@newacrnourl
}

\let\newabbrnourl\newacrnourl

\newcommand*{\@newacrnourl}[4][]{%
  \def\@acro@opts{#1}%
  \def\@glo@shortpl{#3s}%
  \def\@glo@longpl{#4s}%
  \let\@glo@def@longpl\@glo@longpl
  \setkeys{glossentry}{#1}%
% set plural key
  \toks@{#4}%
  \epreto\@acro@opts{plural={\expandonce\@glo@shortpl
    \noexpand\protect\noexpand\index{\the\toks@|hyperpage}},}%
% set first plural key
  \toks@{#4}%
  \epreto\@acro@opts{firstplural={\expandonce\@glo@longpl
    \noexpand\protect\noexpand\index{\the\toks@|hyperpage} 
      (\expandonce\@glo@shortpl)},}%
  \toks@{{#3}{#4}}%
  \ifdefequal{\@glo@def@longpl}{\@glo@longpl}%
  {}%
  {\epreto\@acro@extdesc{ (plural: \expandonce\@glo@longpl)}}%
  \edef\@do@newacronourl{%
    \noexpand\@@newacrnourl
      {\expandonce\@acro@extdesc}%
      {\expandonce\@acro@opts}%
      {#2}%
      \the\toks@
  }%
  \@do@newacronourl
}

\newcommand*{\@@newacrnourl}[5]{%
  \newglossaryentry{#3}{%
    type=acronym,
    sort={#4},
    name={#4\gloindex{#4}},
    text={#4\protect\index{#4|hyperpage}},
    first={#5\protect\index{#4|hyperpage} (#4)},
    description={\makefirstuc{#5}#1},
    user4={#4},%
    #2%
  }%
}

% \newterm[plural]{label}{name}{description}
\newcommand*{\newterm}[4][\admin@term@pl]{%
  \def\admin@term@pl{#3s}%
  \toks@{%
     sort={#3},
     name={\capitalisewords{#3}\gloindex{#3}},
     text={#3\protect\index{#3|hyperpage}},
     description={#4\nopostdesc},%
  }%
  \protected@edef\do@newglossaryentry{%
    \noexpand\newglossaryentry{#2}{%
      type=acronym,%
      \the\toks@
      plural={#1\noexpand\protect\noexpand\index{#3|hyperpage}},
      user4={#3},%
      symbol={#1}%
    }%
  }%
  \do@newglossaryentry
}

\newcommand*{\appindex}[2][hyperpage]{%
  \edef\@doindex{#2\actualchar\string\appname{#2}|#1}%
  \expandafter\index\expandafter{\@doindex}%
}

% \defapp[label]{name}{description}{url}
\newcommand*{\defapp}[4][\admin@app@label]{%
  \def\admin@app@label{#2}%
  \newglossaryentry{#1}{%
    type=acronym,%
    sort={#2},
    name={\appname{#2}\gloindex{#2\actualchar\string\appname{#2}}},
    text={\appname{#2}\protect\index{#2\actualchar\string\appname{#2}|hyperpage}},
    description={#3. URL: \href{#4}{\nolinkurl{#4}}\nopostdesc},%
    user4={\appname{#2}}%
  }%
}

% \defappnourl[label]{name}{description}
\newcommand*{\defappnourl}[3][\admin@app@label]{%
  \def\admin@app@label{#2}%
  \newglossaryentry{#1}{%
    type=acronym,%
    sort={#2},
    name={\appname{#2}\gloindex{#2\actualchar\string\appname{#2}}},
    text={\appname{#2}\protect\index{#2\actualchar\string\appname{#2}|hyperpage}},
    description={#3\nopostdesc},%
    user4={\appname{#2}}%
  }%
}

% \deflang[label]{name}{description}{url}
\newcommand*{\deflang}[4][\admin@app@label]{%
  \def\admin@app@label{#2}%
  \newglossaryentry{#1}{%
    type=acronym,%
    sort={#2},
    name={#2\gloindex{#2}},
    text={#2\protect\index{#2|hyperpage}},
    description={#3. URL: \url{#4}\nopostdesc},%
    user4={#2}%
  }%
}

\renewcommand*{\glossentryname}[1]{%
  \glsdoifexists{#1}%
  {%
    \glsnamefont{\glsnameaccessdisplay{\glsentryname{#1}}{#1}}%
  }%
}

\newglossarystyle{acronyms}{%
  \setglossarystyle{list}%
  \renewcommand*{\glsgroupskip}{}%
  \renewenvironment{theglossary}%
    {\begin{description}}
    {%
      \end{description}%
      \protected@write\@auxout{}{\string\@flow@endacr{\theabsolutepage}}%
    }%
}

% Code optimized for book, but some tweaking may be required
\ifnum\adminmedia=\ADMINbook
  \DeclareRobustCommand{\bookpagebreak}{\par\pagebreak}
  \DeclareRobustCommand{\booklinebreak}{\linebreak}
  \DeclareRobustCommand{\booknewline}{\newline}
  \DeclareRobustCommand{\bookcontinueline}{\continue@symbol\newline}
\else
  \newcommand*{\bookpagebreak}{}
  \newcommand*{\booklinebreak}{}
  \newcommand*{\booknewline}{}
  \newcommand*{\bookcontinueline}{}
\fi
\ifnum\adminmedia=\ADMINreport
  \DeclareRobustCommand{\reportpagebreak}{\par\pagebreak}
  \DeclareRobustCommand*{\reportlinebreak}{\linebreak}
  \DeclareRobustCommand*{\reportnewline}{\newline}
  \DeclareRobustCommand{\reportcontinueline}{\continue@symbol\newline}
\else
  \newcommand*{\reportpagebreak}{}
  \newcommand*{\reportlinebreak}{}
  \newcommand*{\reportnewline}{}
  \newcommand*{\reportcontinueline}{}
\fi
\ifnum\adminmedia=\ADMINscreen
  \newcommand*{\screenpagebreak}{\par\pagebreak}
  \newcommand*{\screenlinebreak}{\linebreak}
  \newcommand*{\screennewline}{\newline}
  \DeclareRobustCommand{\screencontinueline}{\continue@symbol\newline}
  \newcommand*{\screenclearpage}{\clearpage}
\else
  \newcommand*{\screenpagebreak}{}
  \newcommand*{\screenlinebreak}{}
  \newcommand*{\screennewline}{}
  \newcommand*{\screencontinueline}{}
  \newcommand*{\screenclearpage}{}
\fi

\newcommand*{\continueline}{\screencontinueline\bookcontinueline\reportcontinueline}

\newcommand*{\continuenote}{\hypertarget{continuesym}{If} a~line of
code must be typed without any \gls{eol} characters but is too long
to display on the page, the symbol
\glsaccsupp{line-wrap-symbol}{\continuesymbol}\ is used to indicate
a~line wrap. Make sure you don't insert a~line break at that point.}

\newcommand*{\continuesymbol}{$\hookleftarrow$}

\newcommand*{\continue@symbol}{%
 \glsaccsupp{line-wrap-symbol}%
 {%
   \raisebox{-1.6ex}%
   {%
     \makebox[0pt][r]{\hyperlink{continuesym}{\continuesymbol}}%
   }%
 }%
}

% index and display application name with hyperlink to application's
% home page
 \ifnum\adminmedia=\ADMINbook

  \newcommand{\iappnamelink}[3][]{%
   \appname{#2}#1\footnote{\url{#3}}%
   \index{#2\actualchar\appname{#2}}}

  \newcommand{\iperldistro}[3][]{%
    \perldistro{#2}#1\footnote{\url{#3}}%
    \index{#2\actualchar\protect\perldistro{#2}|hyperpage}%
  }

  \newcommand{\itexdistrolink}[3][]{%
  {%
    \texdistro{#2}#3\footnote{\url{#3}}%
    \index{#2\actualchar\protect\texdistro{#2}|hyperpage}%
    \index{TeX Distributions\actualchar\protect\TeX\
Distributions!#2\actualchar\protect\texdistro{#2}|hyperpage}%
  }%
 }

 \else
  \newcommand{\iappnamelink}[3][]{%
   \htmladdnormallink{\appname{#2}}{#3}#1%
   \index{#2\actualchar\appname{#2}|hyperpage}}

  \newcommand{\iperldistro}[3][]{%
   \htmladdnormallink{\perldistro{#2}}{#3}#1%
   \index{#2\actualchar\protect\perldistro{#2}|hyperpage}}

  \newcommand{\itexdistrolink}[3][]{%
  {%
    \htmladdnormallink{\texdistro{#2}}{#3}#1%
    \index{#2\actualchar\texdistro{#2}|hyperpage}%
    \index{TeX Distributions\actualchar\protect\TeX\
Distributions!#2\actualchar\protect\texdistro{#2}|hyperpage}%
  }%
 }

 \fi

\newcommand{\suppmaterial}[2][]{#2#1\footnote{\url{\baseurl/latex/admin/supplemental.html}}}
\newcommand{\suppref}[3][]{#2#1\footnote{\url{\baseurl/latex/admin/#3.html}}}

\newcommand{\objectref}[2]{\htmlref{#1~\ref*{#2}}{#2}}

\newcommand{\xfigureref}[1]{\figurename~\ref{#1}}

\newcommand*{\figuresref}{\@ifstar\@sfiguresref\@figuresref}

\newcommand*{\@figuresref}[1]{%
  \figurename{}s~\displayrefs{#1}\doifbook\@displayrefpages
}

\newcommand*{\@sfiguresref}[1]{%
  \figurename{}s~\displayrefs{#1}%
}

\newcommand*{\tablesref}{\@ifstar\@stablesref\@tablesref}

\newcommand*{\@tablesref}[1]{%
  \tablename{}s~\displayrefs{#1}\doifbook\@displayrefpages
}

\newcommand*{\@stablesref}[1]{%
  \tablename{}s~\displayrefs{#1}%
}

\newcommand*{\sectionsref}[1]{%
  Sections~\displayrefs{#1}%
}

\newcommand*{\refsep}{}
\newcommand*{\lastref}{}
\newcommand*{\dolastref}{}
\newcommand*{\firstref}{}
\newcommand*{\prelastref}{}

\newcommand*{\displayrefsdo}[1]{%
 \ifdefempty{\firstref}{\def\firstref{#1}}{}%
 \refsep
 \dolastref
 \renewcommand{\dolastref}{%
   \renewcommand*{\refsep}{, }%
   \renewcommand*{\prelastref}{ and~}%
   \ref{#1}%
   \renewcommand*{\lastref}{#1}%
}}%

\newcommand*{\displayrefs}[1]{%
  \renewcommand*{\refsep}{}%
  \renewcommand*{\lastref}{}%
  \renewcommand*{\dolastref}{}%
  \renewcommand*{\prelastref}{}%
  \renewcommand*{\firstref}{}%
  \forcsvlist{\displayrefsdo}{#1}%
  \prelastref \dolastref
}

\newcount\@displayrefcount

\newcommand*{\@displayrefpages}{%
  \edef\thefirstref{\getpagerefnumber\firstref}%
  \edef\thelastref{\getpagerefnumber\lastref}%
  \global\advance\@displayrefcount by \@ne\relax
  \label{displayrefcount.\number\@displayrefcount}%
  \edef\thispageref{\getpagerefnumber{displayrefcount.\number\@displayrefcount}}%
  \ifnum\thispageref=\thefirstref
   \relax
  \else
    \ifnum\thefirstref=\thelastref
      \space\vpageref\firstref
    \else
      \ifnum\thelastref=\numexpr(\thefirstref+1)\relax
       \space pages~\pageref\firstref\ and~\pageref\lastref
      \else
       \space pages~\pageref\firstref--\pageref\lastref
      \fi
    \fi
  \fi
}

\newcommand{\xpageref}[2]{\vpageref{#2}}

\newcommand{\figuretop}[1]{\hypertarget{#1top}{}}
\newcommand{\makeimg}[2]{{#2}}
\let\oelig\oe

\newlength\rightmargini
\newlength\rightmarginii
\newlength\rightmarginiii
\newlength\rightmarginiv

\def\itemize{%
  \ifnum\@itemdepth>\thr@@
    \@toodeep
  \else
    \setlength{\rightmargini}{5em}%
    \advance\@itemdepth\@ne
    \edef\@itemitem{labelitem\romannumeral\the\@itemdepth}%
    \expandafter\list\csname\@itemitem\endcsname
     {\def\makelabel##1{\hss\llap{##1}}}%
  \fi
}

\def\enumerate{%
  \ifnum\@enumdepth>\thr@@
    \@toodeep
  \else
    \advance \@enumdepth\@ne
    \edef\@enumctr{enum\romannumeral\the\@enumdepth}%
    \expandafter\list\csname label\@enumctr\endcsname
      {% 
         \usecounter\@enumctr
         \setlength{\rightmargin}%
            {\csname rightmargin\romannumeral\the\@enumdepth\endcsname}%
         \def\makelabel##1{\hss\llap{##1}}%
      }%
  \fi
}

\newcommand{\fwitem}[1]{\item[{\makebox[\labelwidth][l]{#1}}]}

\newenvironment{fwlist}[1]{%
 \begin{list}{}%
 {%
  \settowidth{\labelwidth}{#1}\setlength{\itemindent}{0pt}%
  \setlength{\leftmargin}{\labelwidth}%
  \addtolength{\leftmargin}{\labelsep}%
 }
}%
{%
 \end{list}%
}

\newcounter{exbackref}

\newcommand*{\divhundred}[1]{%
  \ifnum#1<0
    -\divhundred{-#1}%
  \else
    \ifnum\numexpr#1-(#1/100)*100<0
       \number\numexpr(#1)/100-1\relax
    \else
       \number\numexpr((#1)/100)\relax
    \fi
  \fi
}

\newcommand*{\divten}[1]{%
  \ifnum#1<0
    -\divten{-#1}%
  \else
    \ifnum\numexpr#1-(#1/10)*10<0
       \number\numexpr(#1)/10-1\relax
    \else
       \number\numexpr((#1)/10)\relax
    \fi
  \fi
}

\newcommand*{\@pagerangeref}[3]{%
  \vrefpagenum\@tempa{#2}%
  \vrefpagenum\@tempb{#3}%
  \refstepcounter{exbackref}%
  \label{exbackref.\number\c@exbackref}%
  \edef\@tempc{\getpagerefnumber{exbackref.\number\c@exbackref}}%
  \ifx\@tempa\@tempb
    \ifx\@tempa\@tempc
      #1%
    \else
      (p.~\@tempa)%
    \fi
  \else
    \ifx\@tempc\@tempb
      #1%
    \else
      \edef\@tempd{\number\numexpr-1+0\@tempa}%
      \ifx\@tempd\@tempb
        \reftextafter
      \else
        \ifdefstring{\@tempb}{??}%
        {%
          (pp.~\@tempa--\@tempb)%
        }%
        {%
          \ifdefstring{\@tempa}{??}{\def\@tempa{0}}{}%
          \edef\@tempc{\number\numexpr\divten{\@tempb}-\divten{\@tempa}}%
          \ifnum\@tempc=0\relax
            (pp.~\@tempa--\number\numexpr\@tempb-10*\divten\@tempb\relax)%
          \else
          \edef\@tempc{\number\numexpr\divhundred{\@tempb}-\divhundred{\@tempa}}%
            \ifnum\@tempc=0\relax
              (pp.~\@tempa--\number\numexpr\@tempb-100*\divhundred\@tempb\relax)%
            \else
              (pp.~\@tempa--\@tempb)%
            \fi
          \fi
        }%
      \fi
    \fi
  \fi
}

\newcommand{\exampleref}[2][\reftextcurrent]{\objectref{Example}{#2}%
  \ifnum\adminmedia=\ADMINbook\relax
    \space\@pagerangeref{#1}{#2}{endex.#2}%
  \fi
}

\newcommand*{\exerciseref}[2][\reftextcurrent]{%
  \objectref{Exercise}{#2}%
  \ifnum\adminmedia=\ADMINbook\relax
    \space\@pagerangeref{#1}{#2}{endex.#2}%
  \fi
}

\newcommand{\modification}[1]{\textbf{\ttfamily #1}}

\newcommand{\comptexttex}{\texttt{comp.text.tex}~\cite{comptexttex}}
\newcommand{\texhax}{\texttt{texhax}~\cite{texhax}}

 \ifcase\adminmedia
 % report
   \newcommand{\latexnovices}{%
     \htmladdnormallink{\emph{\LaTeX\ for Complete Novices}}%
       {\baseurl/latex/novices/index.html}~\cite{novices}%
   }
   \newcommand{\latexthesis}{%
     \htmladdnormallink{\emph{Using \LaTeX\ to Write a PhD Thesis}}%
       {\baseurl/latex/thesis/index.html}~\cite{thesis}%
   }
 \or
 % screen

   \newcommand{\latexnovices}{%
     \htmladdnormallink{\emph{\LaTeX\ for Complete Novices}}%
       {\baseurl/latex/novices/index.html}~\cite{novices}%
   }%
   \newcommand{\latexthesis}{%
     \htmladdnormallink{\emph{Using \LaTeX\ to Write a PhD Thesis}}%
       {\baseurl/latex/thesis/index.html}~\cite{thesis}%
   }
 \or
 % book

   \newcommand{\latexnovices}{%
     \emph{\LaTeX\ for Complete Novices}~\cite{novices}%
   }

   \newcommand{\latexthesis}{%
     \emph{Using \LaTeX\ to Write a PhD Thesis}~\cite{thesis}%
   }
 \fi

\newcommand*{\novices}[2][\nov@sec@lab]{%
  \def\nov@sec@lab{sec:#2}%
  \htmladdnormallink{Volume~1}{\baseurl/latex/novices/html/#2.html}%
  ~\cite[\S\ref*{nov-#1}]{novices}%
}

\newcommand*{\xrsectionref}[3][\nov@sec@lab]{%
  \def\nov@sec@lab{sec:#3}%
  \S\ref*{nov-#1}%
}

\newcommand*{\thesis}[2][\nov@sec@lab]{%
  \def\nov@sec@lab{sec:#2}%
  \htmladdnormallink{Volume~2}{\baseurl/latex/thesis/html/#2.html}%
  ~\cite[\S\ref*{thesis-#1}]{thesis}%
}

\newcommand{\latexpdfdoc}{%
 \htmladdnormallink{Volume~4}{\baseurl/latex/pdfdoc/}~\cite{pdfdoc}%
}

\newenvironment{resultS*}[1][]%
{\begin{ltxresultS}}%
{\end{ltxresultS}\ignorespacesafterend}

\newenvironment{resultS}[1][]%
{\begin{ltxresultS}}%
{\end{ltxresultS}\ignorespacesafterend}

\newenvironment{outerresult}[1][]%
{\begin{ltxresultM}\ignorespaces}%
{\end{ltxresultM}\ignorespacesafterend}

\newenvironment{innerresult}[1][]%
{\ignorespaces}%
{\ignorespacesafterend}

\newenvironment{result}[1][]%
{%
  \ifinner
    \def\result@env{innerresult}%
  \else
    \def\result@env{outerresult}%
  \fi
  \begin{\result@env}\ignorespaces
}%
{%
  \end{\result@env}\ignorespacesafterend
}

\newcounter{object}

\newenvironment{inlinedef}{\begin{ttfamily}}{\end{ttfamily}}

% \casemedia{A4}{screen}{book}{html}
\newcommand{\casemedia}[4]{%
  \ifcase\adminmedia#1\or#2\or#3\fi%
}

\ifnum\adminmedia=\ADMINscreen
  \newcommand*{\widetablenote}{}
  \newcommand*{\widetableenvname}{table}
  \newenvironment{widetable}[1][htbp]{\begin{table}[#1]\small}{\end{table}}
\else
  \latexhtml
  {
    \newcommand*{\widetableenvname}{sidewaystable}
    \newcommand*{\widetablenote}{\space
      (I've used a~\gls{env-sidewaystable}, described in 
      \novices{sidewaysfloats}, as it's too wide to fit the page)%
    }
   \newenvironment{widetable}[1][]{\begin{sidewaystable}}{\end{sidewaystable}}
  }
  {
    \newcommand*{\widetablenote}{}
    \newcommand*{\widetableenvname}{table}
  }
\fi

\newcommand*{\ctopicfmt}[1]{\textsf{#1}}

\newcommand*{\ctopic}{\@ifstar\s@ctopic\@ctopic}

\newcommand*{\s@ctopic}[1]{%
  \ifbookorother
  {\ctopicfmt{#1}~\cite{topic-#1}}%
  {\htmladdnormallink{\ctopicfmt{#1}}{http://ctan.org/topic/#1}}%
}

\newcommand*{\@ctopic}[2][\@topictext]{%
  \def\@topictext{\ctopicfmt{#2} topic}%
  \ifbookorother
  {\@topictext~\cite{topic-#2}}%
  {\htmladdnormallink{\@topictext}{http://ctan.org/topic/#2}}%
}

\ifnum\adminmedia=\ADMINbook\relax
  \newcommand*{\catalogue}[2]{\dq{#1} section}
\else
  \newcommand*{\catalogue}[2]{%
    \htmladdnormallink{\dq{#1}}{http://mirror.ctan.org/help/Catalogue/bytopic.html\hashsym #2}}
\fi

\newcommand{\download}[2][.tex]{%
\casemedia
 {\htmladdnormallink{download}{\downloadurl/#2#1}}% report
 {\htmladdnormallink{download}{\downloadurl/#2#1}}% screen
 {download\footnote{\url{\downloadurl/#2#1}}}% book
 {\htmladdnormallink{download}{\downloadurl/#2#1}}% html
}

\newcommand{\downloadorview}[2][tex]{%
\casemedia
 {\htmladdnormallink{download}{\downloadurl/#2.#1} or
\htmladdnormallink{view}{\downloadurl/#2.shtml}}% report
 {\htmladdnormallink{download}{\downloadurl/#2.#1} or
\htmladdnormallink{view}{\downloadurl/#2.shtml}}% screen
 {download\footnote{\url{\downloadurl/#2.#1}}}% book
 {\htmladdnormallink{download}{\downloadurl/#2.#1} or
\htmladdnormallink{view}{\downloadurl/#2.shtml}}% html
}

\newcommand{\downloadorviewans}[1]{%
\casemedia
 {\htmladdnormallink{download}{\ansdownloadurl/#1.tex} or
\htmladdnormallink{view}{\ansdownloadurl/#1.shtml}}% report
 {\htmladdnormallink{download}{\ansdownloadurl/#1.tex} or
\htmladdnormallink{view}{\ansdownloadurl/#1.shtml}}% screen
 {download\footnote{\url{\ansdownloadurl/#1.tex}}}% book
 {\htmladdnormallink{download}{\ansdownloadurl/#1.tex} or
\htmladdnormallink{view}{\ansdownloadurl/#1.shtml}}% html
}

\let\@period=.
\let\@comma=,

\newcommand{\@ifnextpunc}[2]{%
 \def\reserved@a{#1}%
 \def\reserved@b{#2}%
 \futurelet\@let@token\@@ifnextpunc
}

\newcommand{\@@ifnextpunc}{%
 \ifx\@let@token\@period
  \let\reserved@b\reserved@a
  \else
   \ifx\@let@token\@comma
    \let\reserved@b\reserved@a
   \fi
 \fi
 \reserved@b
}

\ifnum\adminmedia=\ADMINbook\relax
  \newcommand*{\sampleimage}[1]{%
   \@ifnextpunc{\@sampleimage{#1}}{\@sampleimage{#1}{}}%
  }

  \newcommand*{\@sampleimage}[2]{%
    \file{#1}#2\footnote{\url{\downloadurl/#1}}%
  }
\else
  \newcommand*{\sampleimage}[1]{%
    \href{\downloadurl/#1}{\file{#1}}%
  }
\fi


\newcommand*{\ifbookorother}[2]{%
  \ifnum\adminmedia=\ADMINbook\relax
   #1%
  \else
   #2%
  \fi
}

\newcommand*{\ifscreenorother}[2]{%
  \ifnum\adminmedia=\ADMINscreen\relax
   #1%
  \else
   \ifpdf
    #2%
   \else
    #1%
   \fi
  \fi
}

\newcommand*{\ifreportorother}[2]{%
  \ifnum\adminmedia=\ADMINreport\relax
   #1%
  \else
   #2%
  \fi
}

\newcommand*{\doifscreen}[1]{%
  \ifnum\adminmedia=\ADMINscreen\relax
   #1%
  \fi
}

\newcommand*{\doifreport}[1]{%
  \ifnum\adminmedia=\ADMINreport\relax
   #1%
  \fi
}

\newcommand*{\doifbook}[1]{%
  \ifnum\adminmedia=\ADMINbook\relax
   #1%
  \fi
}

\newcommand*{\doifnotbook}[1]{%
  \ifnum\adminmedia=\ADMINbook
  \else
   #1%
  \fi
}

\newcommand*{\breakenumi}{%
  \xdef\currentenumi{\arabic{enumi}}
  \end{enumerate}
  \begin{enumerate}
  \setcounter{enumi}{\currentenumi}
}

\newcommand*{\latexcaption}{\caption}
\newcommand*{\latexlabel}[1]{\label{#1}}

%\dosidewaysfigure{label}{image}{caption}
\newcommand*{\dosidewaysfigure}[3]{%
  \ifscreenorother
  {%
    \begin{figure}
      \figuretop{#1}
      \rotatebox{90}%
      {%
        \begin{minipage}[c][\textwidth]{0.75\textheight}%
         \centering
         #2% image
         \caption{#3}% caption
         \label{#1}%
        \end{minipage}
      }
    \end{figure}
  }%
  {%
    \begin{sidewaysfigure}
      \figuretop{#1}
      \centering
      #2% image
      \caption{#3}% caption
      \label{#1}%
    \end{sidewaysfigure}
  }%
}

\ifscreenorother
 {\newenvironment*{widefigureenv}{\begin{figure}[p]}{\end{figure}}}%
 {\newenvironment*{widefigureenv}{\begin{sidewaysfigure}}{\end{sidewaysfigure}}}

\newcommand*{\degrees}[1]{\ensuremath{#1^{\circ}}}

\newcommand*{\idxcrossref}[2]{\see{#1}{#2}}

% display and index a counter:
\newcommand{\icounter}[1]{%
\counter{#1}\index{counters!#1\actualchar\counter{#1}|hyperpage}}

% display and index BibTeX:
\newcommand{\iBiBTeX}{\BiBTeX\index{BibTeX\actualchar\BiBTeX|hyperpage}}

% display and index PDFLaTeX:
\newcommand{\iPDFLaTeX}{\PDFLaTeX\index{PDFLaTeX\actualchar\PDFLaTeX|hyperpage}}

% display and index text:
\newcommand{\Index}[1]{#1\index{#1|hyperpage}}

% index monospaced text:
\newcommand{\indextt}[1]{\index{#1\actualchar\texttt{#1}|hyperpage}}

% display and index monospaced text:
\newcommand*{\Indextt}[1]{\texttt{#1}\indextt{#1}}

% display a boolean variable:
\newcommand*{\boolvar}[1]{\texttt{#1}}

% display and index a boolean variable:
\newcommand*{\iboolvar}[1]{%
 \boolvar{#1}%
 \index{#1\actualchar\boolvar{#1} boolean flag}%
}

% display a beamer theme name:
\newcommand*{\theme}[1]{\texttt{#1}}

% display and index a beamer theme name:
\newcommand*{\itheme}[1]{%
 \theme{#1}%
 \index{beamer themes\actualchar\string\cls{beamer} themes!#1\actualchar\theme{#1}|hyperpage}%
}

\newcommand*{\ithemetype}[1]{%
 #1 theme%
 \index{beamer themes\actualchar\string\cls{beamer} themes!#1|hyperpage}%
}

% command definition: display and index
\newcommand*{\cmddef}[1]{\cmdname{#1}\indexComdef{#1}}

% index and display a command name
\newcommand*{\icmdname}[1]{\cmdname{#1}\indexCom{#1}}

\newcommand*{\cmdopt}[1]{\optfmt{#1}}
\newcommand*{\icmdopt}[2]{\cmdopt{#2}%
  \@for\@thiscmd:=#1\do
  {%
    \index{\@thiscmd
\actualchar\protect\cmdname{\@thiscmd}!#2\actualchar\protect\cmdopt{#2}|hyperpage}%
  }%
}

% index an environment definition, but don't display it
\newcommand*{\indexEnvdef}[1]{\index{#1\actualchar\envname{#1} environment|indexdef}}

% index an environment, but don't display it
\newcommand*{\indexEnv}[1]{\index{#1\actualchar\envname{#1}
environment|hyperpage}}

% environment definition: display and index
\newcommand*{\envdef}[1]{\envname{#1}\indexEnvdef{#1}}

% index and display environment
\newcommand*{\ienvname}[1]{\envname{#1}\index{#1\actualchar\envname{#1} environment|hyperpage}}

\newcommand*{\envopt}[1]{\optfmt{#1}}

\newcommand*{\ienvopt}[2]{\envopt{#2}%
  \@for\@thisenv:=#1\do
  {%
    \index{\@thisenv\actualchar\envname{\@thisenv} environment!#2\actualchar\protect\envopt{#2}|hyperpage}%
  }%
}
\newtoks\argsummarytok

\newcommand*{\indexglo}[1]{\textbf{\hyperpage{#1}}}

\DeclareRobustCommand*{\gloindex}[1]{\index{#1|indexglo}}

\newcommand*{\glsi}[1]{%
 \glsdoifexists{#1}{\glshyperlink[\glsentrytext{#1}]{#1}}%
}

\newcommand*{\glsni}{\@ifstar\s@glsni\@glsni}

\newcommand*{\@glsni@add}[1]{\ifglsused{#1}{}{\glsadd{#1}\glsunset{#1}}}

% Don't generate an entry in the index, but add an entry
% to the glossary if this hasn't been used yet.
\newcommand*{\@glsni}[1]{%
 \glsdoifexists{#1}%
 {%
  \@glsni@add{#1}%
  \glshyperlink[\glsentryuseriv{#1}]{#1}%
 }%
}

\newcommand*{\s@glsni}[1]{%
 \glsdoifexists{#1}%
 {%
   \glsentryuseriv{#1}%
 }%
}

\newcommand*{\@noindex}[1]{\@bsphack\begingroup\@sanitize\@index}

\newcommand*{\glsplni}[1]{%
 \glsdoifexists{#1}%
 {%
  \@glsni@add{#1}%
  \begingroup
  \let\index\@noindex
  \glshyperlink[\glsentryplural{#1}]{#1}%
  \endgroup
 }%
}

\newcommand*{\glsnl}[1]{%
 \glsdoifexists{#1}{\glsentryuseriv{#1}}%
}

\DeclareRobustCommand*{\indexuservi}[1]{%
  \expandafter\index\expandafter{\glsentryuservi{#1}|hyperpage}%
}

\DeclareRobustCommand*{\indexdefuservi}[1]{%
  \expandafter\index\expandafter{\glsentryuservi{#1}|indexdef}%
}

\newcommand*{\lkernel}{\LaTeX\ Kernel}
\newcommand*{\texprimitive}{\TeX\ primitive}
\newcommand*{\pdftexprimitive}{\PDFTeX\ primitive}

% \defgcs[extra][label]{name}{arg syntax}{location of definition}{summary}{arg
% summary}
\newcommand{\defgcs}[1][]{%
 \def\@gcs@extra{#1}%
 \@defgcs
}

\newcommand{\@defgcs}[6][\gcslabel]{%
 \def\gcslabel{#2}%
 \argsummarytok{#6}%
 \expandafter\newglossaryentry\expandafter
   {#1}%
   {%
     sort={#2},
     name={\cmdname{#2}},
     text={\protect\cmdname{#2}\indexuservi{#1}},
     user1={#3},
     user2={#4},
     description={#5},
     user3={\the\argsummarytok},
     user4={\protect\cmdname{#2}},
     user5={\@gcs@extra},
     user6={#2\actualchar\protect\cmdname{#2}}
   }%
}

% \defgchar[sort]{label}{char}{arg syntax}{location}{summary}{arg summary}
\newcommand{\defgchar}[7][\@sortkey]{%
  \def\@sortkey{#3}%
  \newglossaryentry{#2}%
   {
     name={\texttt{#3}},
     sort={#1},
     text={\protect\texttt{#3}%
      \indexuservi{#2}},
     user1={#4},
     user2={#5},
     user3={#7},
     user4={\texttt{#3}},
     description={#6},
     user6={#3\actualchar\texttt{#3}}
   }%
}

% \defgchildchar{label}{parent}{char}{summary}
\newcommand{\defgchildchar}[4]{%
  \newglossaryentry{#1}%
  {%
     parent={#2},
     description={#4},
     first={\texttt{#3}%
      \indexuservi{#1}},
     text={\protect\texttt{#3}%
      \indexuservi{#1}},
     user4={\texttt{#3}},
     user6={#3\actualchar\texttt{#3}}
  }%
}

% \defgxchildchar{label}{parent}{char}{summary}{extra}
\newcommand{\defgxchildchar}[5]{%
  \newglossaryentry{#1}%
  {%
     parent={#2},
     description={#4},
     first={\texttt{#3}%
      \indexuservi{#1}%
      \protect\index{#5|hyperpage}%
     },
     text={\protect\texttt{#3}%
      \indexuservi{#1}%
      \protect\index{#5|hyperpage}%
     },
     user4={\texttt{#3}},
     user6={#3\actualchar\texttt{#3}}
  }%
}

% \defgactivechar[sort]{label}{cs}{arg syntax}{location}{summary}{arg summary}
\newcommand{\defgactivechar}[7][]{%
  \ifstrempty{#1}
  {%
    \def\@gcs@sort{#3}%
  }%
  {%
    \def\@gcs@sort{#1}%
  }%
  \protected@edef\@do@newglossaryentry{%
     \noexpand\newglossaryentry{#2}%
     {%
        name={\noexpand\texttt{\noexpand\protect\noexpand#3}},
        sort={\@gcs@sort},
%        first={\noexpand\texttt{#3}%
%         \indexdefuservi{#2}},
        text={\noexpand\protect\noexpand\texttt{#3}%
         \indexuservi{#2}},
        user1={#4},
        user2={#5},
        user3={#7},
        user4={\noexpand\texttt{#3}},
        description={#6},
        user6={\expandonce\@gcs@sort \actualchar\texttt{\protect#3}}
      }%
   }%
   \@do@newglossaryentry
}

% \defgchildactivechar{label}{parent}{cs}{summary}
\newcommand{\defgchildactivechar}[4]{%
  \newglossaryentry{#1}%
  {%
     parent={#2},
     first={\texttt{#3}%
      \indexuservi{#1}},
     text={\protect\texttt{#3}%
      \indexuservi{#1}},
     user4={\texttt{#3}},
     description={#4},
     user6={#3\actualchar\texttt{#3}}
   }%
}

% \defgactivecharcs[sort]{label}{cs}{arg syntax}{location}{summary}{arg summary}
\newcommand{\defgactivecharcs}[7][]{%
  \ifstrempty{#1}
  {%
    \def\@gcs@sort{#3}%
  }%
  {%
    \def\@gcs@sort{#1}%
  }%
  \protected@edef\@do@newglossaryentry{%
     \newglossaryentry{#2}%
     {%
        name={\noexpand\cmdname{\noexpand\protect\noexpand#3}},
        sort={\backslashsym#1},
%        first={\noexpand\cmdname{#3}%
%         \indexdefuservi{#2}},
        text={\noexpand\protect\noexpand\cmdname{#3}%
         \indexuservi{#2}},
        user1={#4},
        user2={#5},
        user3={#7},
        user4={\noexpand\cmdname{#3}},
        description={#6},
        user6={\backslashsym#1\actualchar\protect\cmdname{\string#3}}
      }%
   }%
   \@do@newglossaryentry
}

% \defgidxactivechar[sort]{label}{cs}{arg syntax}{location}{summary}{arg summary}
\newcommand{\defgidxactivechar}[7][]{%
  \ifstrempty{#1}
  {%
    \def\@gcs@sort{#3}%
  }%
  {%
    \def\@gcs@sort{#1}%
  }%
  \toks@{#6}%
  \protected@edef\@do@newglossaryentry{%
    \newglossaryentry{#2}%
     {%
       name={\noexpand\texttt{\noexpand#3}},
       sort={"\expandonce\@gcs@sort},
%       first={\noexpand\protect\noexpand\texttt{\noexpand#3}%
%        \indexdefuservi{#2}},
       text={\noexpand\protect\noexpand\texttt{\noexpand#3}%
        \indexuservi{#2}},
       user1={#4},
       user2={#5},
       user3={#7},
       user4={\noexpand\texttt{\noexpand#3}},
       description={\the\toks@},
       user6={"\expandonce\@gcs@sort \actualchar\texttt{\string#3}}
   }%
  }%
  \@do@newglossaryentry
}

% \defgidxactivecharaccsupp[sort]{label}{cs}{arg
% syntax}{location}{summary}{arg summary}{access text}
\newcommand{\defgidxactivecharaccsupp}[8][]{%
  \ifstrempty{#1}
  {%
    \def\@gcs@sort{#3}%
  }%
  {%
    \def\@gcs@sort{#1}%
  }%
  \toks@{#6}%
  \protected@edef\@do@newglossaryentry{%
    \newglossaryentry{#2}%
     {%
       name={\noexpand\texttt{\noexpand#3}},
       sort={"\expandonce\@gcs@sort},
       text={\noexpand\protect\noexpand\texttt{\noexpand#3}%
        \indexuservi{#2}},
       textaccess={#8},%
       firstaccess={#8},%
       user1={#4},
       user2={#5},
       user3={#7},
       user4={\glstextaccessdisplay{\noexpand\texttt{\noexpand#3}}{#2}},
       description={\the\toks@},
       user6={"\expandonce\@gcs@sort \actualchar\texttt{\string#3}}
   }%
  }%
  \@do@newglossaryentry
}

% \defgchildidxactivechar{label}{parent}{cs}{summary}
\newcommand{\defgchildidxactivechar}[4]{%
  \newglossaryentry{#1}%
  {%
     parent={#2},
     name={\noexpand\texttt{\noexpand#3}},
     first={\noexpand\texttt{\noexpand#3}%
      \indexuservi{#1}},
     text={\protect\texttt{\noexpand#3}%
      \indexuservi{#1}},
     user4={\texttt{#3}},
     description={#4},
     user6={"#3\actualchar\texttt{\string#3}}
   }%
}

% \defgidxactivecharcs{label}{charcs}{arg syntax}{location}{summary}{arg summary}
\newcommand{\defgidxactivecharcs}[6]{%
  \newglossaryentry{#1}%
   {%
     name={\cmdname{\protect#2}},
     sort={"\backslashsym"#2},
%     first={\cmdname{#2}%
%      \indexdefuservi{#1}},
     text={\protect\cmdname{#2}%
      \indexuservi{#1}},
     user1={#3},
     user2={#4},
     user3={#6},
     user4={\cmdname{#2}},
     description={#5},
     user6={"\backslashsym"#2\actualchar\protect\cmdname{\string#2}}
   }%
}

% \defgsymcs[label]{sym}{syntax}{location}{summary}{arg summary}
\newcommand{\defgsymcs}[6][\gcslabel]{%
 \def\gcslabel{#2}%
 \argsummarytok{#6}%
 \expandafter\newglossaryentry\expandafter
   {#1}%
   {%
     name={\cmdname{#2}},
     sort={\backslashsym#2},
%     first={\protect\cmdname{#2}%
%       \indexdefuservi{#1}},
     text={\protect\cmdname{#2}%
       \indexuservi{#1}},
     user1={#3},
     user2={#4},
     description={#5},
     user3={\the\argsummarytok},
     user4={\protect\cmdname{#2}},
     user5={},
     user6={\backslashsym#2\actualchar\protect\cmdname{#2}}
   }%
}

% \defgsymcsaccsupp[label]{sym}{syntax}{location}{summary}{arg
% summary}{access}
\newcommand{\defgsymcsaccsupp}[7][\gcslabel]{%
 \def\gcslabel{#2}%
 \argsummarytok{#6}%
 \expandafter\newglossaryentry\expandafter
   {#1}%
   {%
     name={\cmdname{#2}},
     sort={\backslashsym#2},
     text={\protect\cmdname{#2}%
       \indexuservi{#1}},
     textaccess={#7},%
     firstaccess={#7},%
     user1={#3},
     user2={#4},
     description={#5},
     user3={\the\argsummarytok},
     user4={\cmdname{#2}},
     user4={\glstextaccessdisplay{\cmdname{#2}}{#1}},
     user5={},
     user6={\backslashsym#2\actualchar\protect\cmdname{#2}}
   }%
}

% \defgchildsymcs{label}{parent}{cs}{summary}
\newcommand{\defgchildsymcs}[4]{%
  \newglossaryentry{#1}%
  {%
     parent={#2},
     name={\cmdname{#3}},
     sort={\backslashsym#3},
     first={\noexpand\cmdname{#3}%
      \indexuservi{#1}},
     text={\protect\cmdname{\noexpand#3}%
      \indexuservi{#1}},
     user4={\cmdname{#3}},
     description={#4},
     user6={\backslashsym#3\actualchar\protect\cmdname{#3}}
   }%
}

% \defgenv[label]{env name}{arg syntax}{location}{summary}{arg
% summary}
\newcommand{\defgenv}[6][\gcslabel]{%
 \def\gcslabel{env-#2}%
 \argsummarytok{#6}%
 \expandafter\newglossaryentry\expandafter
   {#1}%
   {%
     sort={#2},
     name={\protect\cmdname{begin}\protect\texttt{\protect\marg{#2}}},
     text={\protect\envname{#2}%
        \indexuservi{#1}},
     user1={#3},
     user2={#4},
     description={#5},
     user3={\the\argsummarytok},
     user4={\protect\envname{#2}},
     user5={},
     user6={#2\actualchar\envname{#2} environment}
   }%
}

% arara directive
% \defarara[label]{name}{arg syntax}{summary}{arg summary}
\newcommand{\defarara}[5][\gcslabel]{%
 \def\gcslabel{arara.#2}%
 {%
  \def\vbarsym{\string"\string|}%
  \def\exclamsym{\string"\string!}%
  \xdef\araraindex{#2}%
 }%
 \def\araraactualindex{#2}%
 \@onelevel@sanitize\araraactualindex
 \argsummarytok{#5}%
 \expandafter\newglossaryentry\expandafter
   {#1}%
   {%
     sort={\araraindex},
     name={\protect\texttt{#2}},
     text={\protect\texttt{#2}%
        \indexuservi{#1}},
     user1={#3},
     user2={\appname{arara} directive},
     description={#4},
     user3={\the\argsummarytok},
     user4={\protect\texttt{#2}},
     user5={},
     user6={arara directives\actualchar\string\appname{arara} directives!\araraindex\actualchar\texttt{\araraactualindex}}
   }%
}

\newlength\summaryskip
\newlength\summarypostskip
\newlength\summarypredescskip
\newlength\summarypostnameskip

\setlength{\summaryskip}{2ex plus .5ex minus .5ex}
\setlength{\summarypostskip}{.5ex plus .1ex minus .1ex}
\setlength{\summarypredescskip}{1ex plus .5ex minus .5ex}
\setlength{\summarypostnameskip}{1ex plus .25ex minus .25ex}

\newglossarystyle{summary}% Summary glossary style
{%
  \setglossarystyle{inline}%
  \renewcommand{\glossaryheader}{\raggedright\enablefoothooks
      \ifnum\adminmedia=\ADMINbook\relax
        \setleftfootmark{\dosummarytopmark}%
        \setrightfootmark{\dosummarybottommark}%
      \fi
    }%
  \renewcommand{\glsgroupheading}[1]{%
    \gls@inlinesep\def\gls@inlinesep{}%
    \noindent\makebox[\linewidth][c]%
    {%
      \ifnum\adminmedia=\ADMINbook
        \textbf{\glsgetgrouptitle{##1}}%
      \else
        \protected@write\@auxout{}{\string\gls@header{##1}}%
        \hypertarget{gls:##1}{\textbf{\glsgetgrouptitle{##1}}}%
      \fi
    }%
    \\*[\summarypostskip]%
  }%
  \renewcommand{\glsinlinenameformat}[2]{%
    \def\glsentrylabel{##1}%
    \glstarget{##1}{##2}\discretionary{}{}{}\glsentryuseri{##1}%
    \gloindex{\glsentryuservi{##1}}%
    \dosummarymark{\glsuseriv{##1}}%
    \ifcsempty{glo@##1@userii}%
    {%
      \\*[\summarypredescskip]%
    }%
    {%
      \\*[\summarypostnameskip]%
      \textbf{Defined in: }\glsentryuserii{##1}.%
      \par
      \vspace{\summarypredescskip}%
    }%
  }%
  \renewcommand*{\glsinlinesubnameformat}[2]{\glstarget{##1}{\strut}}%
  \renewcommand{\glsinlinedescformat}[3]{##1%
    \doifnotzero{##3}{\space[\S##3]}\glsentryuserv{\glsentrylabel}}%
  \renewcommand{\glsinlineseparator}{%
    \penalty\@medpenalty
    \par
    \vskip\summaryskip
  }%
  \renewcommand{\glsinlinesubseparator}{;\space}%
  \renewcommand{\glsinlinesubdescformat}[3]{##1%
    \doifnotzero{##3}{\space[\S##3]}}%
  \renewcommand{\glsinlineparentchildseparator}{}%
  \renewcommand{\glsinlinepostchild}{.}%
  \renewcommand{\glspostinline}{%
    \protected@write\@auxout{}{\string\@flow@endsummary{\theabsolutepage}}%
  }%
}

\DeclareRobustCommand*{\doifnotzero}[2]{%
 {%
  \def\glossaryentrynumbers##1{##1}%
  \def\glsnumberformat##1{##1}%
  \def\setentrycounter[##1]##2{}%
  \def\relax{}%
  \protected@xdef\admin@tmp{#1}%
 }%
 \ifdefstring{\admin@tmp}{0}{}{#2}%
}

\newcommand*{\nxipagestyle}[1]{\protect\ipagestyle{#1}}

\newcommand*{\nxiappopt}{\protect\iappopt}

\newcommand*{\nxisty}[1]{\protect\isty{#1}}
\newcommand*{\nxicls}[1]{\protect\icls{#1}}
\newcommand*{\nxitikzlibrary}[1]{\protect\itikzlibrary{#1}}
\newcommand*{\nxiappname}[1]{\protect\iappname{#1}}

\newcommand*{\nxcmddef}{\protect\cmddef}

\newcommand*{\nxglsi}{\protect\glsi}
\newcommand*{\nxglsni}{\protect\glsni}
\newcommand*{\nxglsplni}{\protect\glsplni}

\newcommand*{\nxglslink}[2]{\protect\glslink{#1}{#2}}

\newcommand*{\nxGls}{\protect\Gls}
\newcommand*{\nxgls}{\protect\gls}
\newcommand*{\nxglspl}{\protect\glspl}

\newcommand*{\nxicounter}{\protect\icounter}

\newcommand*{\nxiatsym}{\protect\iatsym}

\newcommand{\dohtmlcolorbox}[2][]{#2}

\newcommand*{\ocaron}{\v{o}}
\newcommand*{\ocedilla}{\c{o}}
\newcommand*{\obarunder}{\b{o}}
\newcommand*{\odotunder}{\d{o}}
\newcommand*{\omacron}{\={o}}
\newcommand*{\odoubleacute}{\H{o}}
\newcommand*{\odotover}{\.{o}}
\newcommand*{\otilde}{\~{o}}
\newcommand*{\ocircum}{\^{o}}
\newcommand*{\obreve}{\u{o}}
\DeclareRobustCommand*{\oumlaut}{\"{o}}
\newcommand*{\uring}{\r{u}}
\newcommand*{\rbarunder}{\b{r}}
\newcommand*{\xytie}{\t{xy}}
\newcommand*{\dotlessj}{\j}

\newcommand{\indexEPS}{%
 \index{Encapsulated PostScript (EPS) file|hyperpage}%
 \index{file formats!eps\actualchar\texttt{.eps}|hyperpage}%
}

\newcommand{\indexDVI}{%
  \index{Device Independent (DVI) file|hyperpage}%
  \index{file formats!dvi\actualchar\texttt{.dvi}|hyperpage}%
}

\newcommand{\indexPDF}{%
  \index{Portable Document Format (PDF) file|hyperpage}%
  \index{file formats!pdf\actualchar\texttt{.pdf}|hyperpage}%
}

\newcommand{\indexTOC}{%
  \index{table of contents file (.toc)\actualchar table of contents file (\texttt{.toc})|hyperpage}%
  \index{file formats!toc\actualchar\texttt{.toc}|hyperpage}%
}

\newcommand{\indexLOF}{%
  \index{list of figures file (.lof)\actualchar list of figures file (\texttt{.lof})|hyperpage}%
  \index{file formats!lof\actualchar \texttt{.lof}|hyperpage}%
}

\newcommand{\indexLOT}{%
  \index{list of tables file (.lot)\actualchar list of tables file (\texttt{.lot})|hyperpage}%
  \index{file formats!lot\actualchar \texttt{.lot}|hyperpage}%
}

\newcommand{\indexLOG}{%
  \index{log file (.log)\actualchar log file (\texttt{.log})|hyperpage}%
  \index{file formats!log\actualchar \texttt{.log}|hyperpage}%
}

\newcommand{\indexAUX}{%
  \index{auxiliary file (.aux)\actualchar auxiliary file (\texttt{.aux})|hyperpage}%
  \index{file formats!aux\actualchar \texttt{.aux}|hyperpage}%
}

\newcommand{\iauxfile}{auxiliary (\texttt{.aux}) file\indexAUX}

\newcommand{\indexDBTEX}{%
  \index{datatool file (.dbtex)\actualchar \sty{datatool} file (\texttt{.dbtex})|hyperpage}%
  \index{file formats!dbtex\actualchar \texttt{.dbtex}|hyperpage}%
}

\newcommand{\idbtexfile}{\sty{datatool} (\texttt{.dbtex}) file\indexDBTEX}

\newcommand{\nxidbtexfile}{\noexpand\idbtexfile}

\newcommand{\indexCSV}{%
  \index{comma-separated variable file (.csv)\actualchar comma-separated
variable file (\texttt{.csv})|hyperpage}%
  \index{file formats!csv\actualchar \texttt{.csv}|hyperpage}%
}

\newcommand{\indexXLS}{%
  \index{Excel spreadsheet (.xls)\actualchar Excel spreadsheet (\texttt{.xls})|hyperpage}%
  \index{file formats!xls\actualchar \texttt{.xls}|hyperpage}%
}

\newcommand{\indexODS}{%
  \index{Open Document Spreadsheet (.ods)\actualchar Open Document Spreadsheet
(\texttt{.ods})|hyperpage}%
  \index{file formats!ods\actualchar \texttt{.ods}|hyperpage}%
}

\newcommand{\indexODT}{%
  \index{Open Document Format (.odt)\actualchar Open Document Format
(\texttt{.odt})|hyperpage}%
  \index{file formats!odt\actualchar \texttt{.odt}|hyperpage}%
}
\newcommand{\iodtfile}{\texttt{.odt}\indexODT}

\newcommand{\indexSYNCTEX}{%
  \index{synctex file (.synctex.gz)\actualchar synctex file (\texttt{.synctex.gz})|hyperpage}%
  \index{file formats!synctex.gz\actualchar \texttt{.synctex.gz}|hyperpage}%
}

\newcommand{\indexCLS}{%
  \index{class files (.cls)\actualchar class files (\texttt{.cls})|hyperpage}%
  \index{file formats!cls\actualchar \texttt{.cls}|hyperpage}%
}

\newcommand{\indexSTY}{%
  \index{packages files (.sty)\actualchar package files (\texttt{.sty})|hyperpage}%
  \index{file formats!sty\actualchar \texttt{.sty}|hyperpage}%
}

\newcommand{\indexTDF}{%
  \index{ticket definition files (.tdf)\actualchar ticket definition
files (\texttt{.tdf})|hyperpage}%
  \index{file formats!tdf\actualchar \texttt{.tdf}|hyperpage}%
}

\newcommand{\indexJDR}{%
  \index{file formats!jdr\actualchar \texttt{.jdr}|hyperpage}%
}

\newcommand{\ijdrfile}{\texttt{.jdr}\indexJDR}

\newcommand{\nxijdrfile}{\noexpand\ijdrfile}

\newcommand{\indexAJR}{%
  \index{file formats!ajr\actualchar \texttt{.ajr}|hyperpage}%
}

\newcommand{\iajrfile}{\texttt{.ajr}\indexAJR}

\newcommand{\nxiajrfile}{\noexpand\iajrfile}

\newcommand{\indexSOC}{%
  \index{list or summary of changes file (.soc)\actualchar list or summary of changes file
(\texttt{.soc})|hyperpage}%
  \index{file formats!soc\actualchar \texttt{.soc}|hyperpage}%
}
\newcommand{\isocfile}{\texttt{.soc}\indexSOC}

\newcommand*{\TO}{\ensuremath{\Rightarrow}}

\DeclareRobustCommand*{\extraindexaux}{%
 \index{file formats!aux\actualchar \texttt{.aux}|indexdef}%
}


\DeclareRobustCommand*{\extraindexsentencespacing}{%
 \index{spacing!French|indexdef}%
 \index{spacing!English|indexdef}%
 \index{French spacing|idxcrossref{spacing, French}}%
 \index{English spacing|idxcrossref{spacing, English}}%
 \index{inter-sentence spacing|idxcrossref{spacing, inter-sentence}}%
}

\DeclareRobustCommand*{\extraindexshortlong}{%
  \index{short command|idxcrossref{command, short}}%
  \index{long command|idxcrossref{command, long}}%
  \index{command!long|indexdef}%
}

\DeclareRobustCommand*{\extraindexfragile}{%
 \index{moving argument|idxcrossref{argument, moving}}%
 \index{argument!moving|indexdef}%
 \index{fragile command|idxcrossref{command, fragile}}%
}

\DeclareRobustCommand*{\extraindexoptional}{%
  \index{optional argument|idxcrossref{argument, optional}}%
}

\DeclareRobustCommand*{\extraindexmandatory}{%
  \index{mandatory argument|idxcrossref{argument, mandatory}}%
}

\DeclareRobustCommand*{\extraindexrobust}{%
  \index{robust command|idxcrossref{command, robust}}%
}

\DeclareRobustCommand*{\extraindexarg}{%
  \index{parameter|idxcrossref{argument}}%
}

\DeclareRobustCommand*{\extraindexgroup}{%
  \index{scope|idxcrossref{group}}%
}

\DeclareRobustCommand*{\extraindexterminal}{%
  \index{command prompt|idxcrossref{terminal}}%
}

\newcommand*{\historyitem}[1]{\section*{#1}}

\ifnum\adminmedia=\ADMINbook
  \newcommand*{\backcoverheading}{%
    \newpage\thispagestyle{empty}%
    \section*{Back Cover Text}%
  }
\else
  \newcommand*{\backcoverheading}{\chapter*{Back Cover Text}}
\fi

\newcommand*{\glstoc}[1]{#1}

\ifcase\adminmedia
% report
 \externaldocument[nov-]{../novices/novices-report}
 \externaldocument[thesis-]{../thesis/thesis-report}
\or
% screen
 \externaldocument[nov-]{../novices/novices-screen}
 \externaldocument[thesis-]{../thesis/thesis-screen}
\or 
% book
 \externaldocument[thesis-]{../thesis/thesis-book}
 \externaldocument[nov-]{../novices/novices-book}
\fi

% Define sample data bases:


\expandafter\newtoks\csname dtlkeys@books\endcsname
\csname dtlkeys@books\endcsname={%
%
% header block for column 1
\db@plist@elt@w %
\db@col@id@w 1%
\db@col@id@end@ %
\db@key@id@w id%
\db@key@id@end@ %
\db@type@id@w 1%
\db@type@id@end@ %
\db@header@id@w Id%
\db@header@id@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 2
\db@plist@elt@w %
\db@col@id@w 2%
\db@col@id@end@ %
\db@key@id@w title%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w Title%
\db@header@id@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 3
\db@plist@elt@w %
\db@col@id@w 3%
\db@col@id@end@ %
\db@key@id@w author%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w Author%
\db@header@id@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 4
\db@plist@elt@w %
\db@col@id@w 4%
\db@col@id@end@ %
\db@key@id@w format%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w Format%
\db@header@id@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 5
\db@plist@elt@w %
\db@col@id@w 5%
\db@col@id@end@ %
\db@key@id@w price%
\db@key@id@end@ %
\db@type@id@w 2%
\db@type@id@end@ %
\db@header@id@w Price (\pounds)%
\db@header@id@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
\db@plist@elt@end@ %
}%

\expandafter\newtoks\csname dtldb@books\endcsname

\csname dtldb@books\endcsname={%
%
% Start of row 1
\db@row@elt@w %
\db@row@id@w 1%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 1%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w The Adventures of Duck and Goose%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Sir Quackalot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w paperback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 10.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 1
\db@row@id@w 1%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 2
\db@row@elt@w %
\db@row@id@w 2%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 2%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w The Return of Duck and Goose%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Sir Quackalot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w paperback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 11.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 2
\db@row@id@w 2%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 3
\db@row@elt@w %
\db@row@id@w 3%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 3%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w More Fun with Duck and Goose%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Sir Quackalot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w paperback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 12.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 3
\db@row@id@w 3%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 4
\db@row@elt@w %
\db@row@id@w 4%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 4%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Duck and Goose on Holiday%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Sir Quackalot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w paperback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 11.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 4
\db@row@id@w 4%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 5
\db@row@elt@w %
\db@row@id@w 5%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 5%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w The Return of Duck and Goose%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Sir Quackalot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w hardback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 19.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 5
\db@row@id@w 5%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 6
\db@row@elt@w %
\db@row@id@w 6%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 6%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w The Adventures of Duck and Goose%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Sir Quackalot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w hardback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 18.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 6
\db@row@id@w 6%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 7
\db@row@elt@w %
\db@row@id@w 7%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 7%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w My Friend is a Duck%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w A. Parrot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w paperback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 14.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 7
\db@row@id@w 7%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 8
\db@row@elt@w %
\db@row@id@w 8%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 8%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Annotated Notes on the `Duck and Goose' chronicles%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Prof Macaw%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w ebook%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 8.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 8
\db@row@id@w 8%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 9
\db@row@elt@w %
\db@row@id@w 9%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 9%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w `Duck and Goose' Cheat Sheet for Students%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Polly Parrot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w ebook%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 5.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 9
\db@row@id@w 9%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 10
\db@row@elt@w %
\db@row@id@w 10%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 10%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w `Duck and Goose': an allegory for modern times?%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Bor Ing%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w hardback%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 59.99%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% End of row 10
\db@row@id@w 10%
\db@row@id@end@ %
\db@row@elt@end@ %
}%

\expandafter\newcount\csname dtlrows@books\endcsname
\csname dtlrows@books\endcsname=10\relax

\expandafter\newcount\csname dtlcols@books\endcsname
\csname dtlcols@books\endcsname=5\relax

\expandafter
 \gdef\csname dtl@ci@books@id\endcsname{1}%
\expandafter
 \gdef\csname dtl@ci@books@title\endcsname{2}%
\expandafter
 \gdef\csname dtl@ci@books@author\endcsname{3}%
\expandafter
 \gdef\csname dtl@ci@books@format\endcsname{4}%
\expandafter
 \gdef\csname dtl@ci@books@price\endcsname{5}%

\newcommand*\books{books}

\expandafter\global\expandafter
\newtoks\csname dtlkeys@people\endcsname
\expandafter\global
 \csname dtlkeys@people\endcsname={%
%
% header block for column 1
\db@plist@elt@w %
\db@col@id@w 1%
\db@col@id@end@ %
\db@key@id@w id%
\db@key@id@end@ %
\db@type@id@w 1%
\db@type@id@end@ %
\db@header@id@w id%
\db@header@id@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 2
\db@plist@elt@w %
\db@col@id@w 2%
\db@col@id@end@ %
\db@key@id@w forenames%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w forenames%
\db@header@id@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 3
\db@plist@elt@w %
\db@col@id@w 3%
\db@col@id@end@ %
\db@key@id@w surname%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w surname%
\db@header@id@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 4
\db@plist@elt@w %
\db@col@id@w 4%
\db@col@id@end@ %
\db@key@id@w title%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w title%
\db@header@id@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 5
\db@plist@elt@w %
\db@col@id@w 5%
\db@col@id@end@ %
\db@key@id@w address1%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w address1%
\db@header@id@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 6
\db@plist@elt@w %
\db@col@id@w 6%
\db@col@id@end@ %
\db@key@id@w address2%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w address2%
\db@header@id@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 7
\db@plist@elt@w %
\db@col@id@w 7%
\db@col@id@end@ %
\db@key@id@w town%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w town%
\db@header@id@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 8
\db@plist@elt@w %
\db@col@id@w 8%
\db@col@id@end@ %
\db@key@id@w county%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w county%
\db@header@id@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 9
\db@plist@elt@w %
\db@col@id@w 9%
\db@col@id@end@ %
\db@key@id@w country%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w country%
\db@header@id@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 10
\db@plist@elt@w %
\db@col@id@w 10%
\db@col@id@end@ %
\db@key@id@w postcode%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w postcode%
\db@header@id@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 11
\db@plist@elt@w %
\db@col@id@w 11%
\db@col@id@end@ %
\db@key@id@w subscribed%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w subscribed%
\db@header@id@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 12
\db@plist@elt@w %
\db@col@id@w 12%
\db@col@id@end@ %
\db@key@id@w gender%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w gender%
\db@header@id@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 13
\db@plist@elt@w %
\db@col@id@w 13%
\db@col@id@end@ %
\db@key@id@w dob%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w dob%
\db@header@id@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
\db@plist@elt@end@ %
}%
\expandafter\global\expandafter
\newtoks\csname dtldb@people\endcsname
\expandafter\global
\csname dtldb@people\endcsname={%
%
% Start of row 1
\db@row@elt@w %
\db@row@id@w 1%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 1%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Polly%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Parrot%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w Miss%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 42 The Lane%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% Column 6
\db@col@id@w 6%
\db@col@id@end@ %
\db@col@elt@w \@dtlnovalue %
\db@col@elt@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
% Column 7
\db@col@id@w 7%
\db@col@id@end@ %
\db@col@elt@w Some Town%
\db@col@elt@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
% Column 8
\db@col@id@w 8%
\db@col@id@end@ %
\db@col@elt@w Noshire%
\db@col@elt@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
% Column 9
\db@col@id@w 9%
\db@col@id@end@ %
\db@col@elt@w gb%
\db@col@elt@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
% Column 10
\db@col@id@w 10%
\db@col@id@end@ %
\db@col@elt@w AB1 2XY%
\db@col@elt@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
% Column 11
\db@col@id@w 11%
\db@col@id@end@ %
\db@col@elt@w true%
\db@col@elt@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
% Column 12
\db@col@id@w 12%
\db@col@id@end@ %
\db@col@elt@w female%
\db@col@elt@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
% Column 13
\db@col@id@w 13%
\db@col@id@end@ %
\db@col@elt@w 1970-12-31%
\db@col@elt@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
% End of row 1
\db@row@id@w 1%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 2
\db@row@elt@w %
\db@row@id@w 2%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 2%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Mabel%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Canary%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w Mrs%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 24 The Street%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% Column 6
\db@col@id@w 6%
\db@col@id@end@ %
\db@col@elt@w Some Village%
\db@col@elt@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
% Column 7
\db@col@id@w 7%
\db@col@id@end@ %
\db@col@elt@w Some Town%
\db@col@elt@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
% Column 8
\db@col@id@w 8%
\db@col@id@end@ %
\db@col@elt@w Noshire%
\db@col@elt@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
% Column 9
\db@col@id@w 9%
\db@col@id@end@ %
\db@col@elt@w gb%
\db@col@elt@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
% Column 10
\db@col@id@w 10%
\db@col@id@end@ %
\db@col@elt@w AB1 2YZ%
\db@col@elt@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
% Column 11
\db@col@id@w 11%
\db@col@id@end@ %
\db@col@elt@w false%
\db@col@elt@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
% Column 12
\db@col@id@w 12%
\db@col@id@end@ %
\db@col@elt@w female%
\db@col@elt@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
% Column 13
\db@col@id@w 13%
\db@col@id@end@ %
\db@col@elt@w 1968-01-23%
\db@col@elt@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
% End of row 2
\db@row@id@w 2%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 3
\db@row@elt@w %
\db@row@id@w 3%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 3%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Z\"oe%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Zebra%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w Ms%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 856 The Avenue%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% Column 6
\db@col@id@w 6%
\db@col@id@end@ %
\db@col@elt@w \@dtlnovalue %
\db@col@elt@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
% Column 7
\db@col@id@w 7%
\db@col@id@end@ %
\db@col@elt@w Some City%
\db@col@elt@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
% Column 8
\db@col@id@w 8%
\db@col@id@end@ %
\db@col@elt@w CA%
\db@col@elt@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
% Column 9
\db@col@id@w 9%
\db@col@id@end@ %
\db@col@elt@w us%
\db@col@elt@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
% Column 10
\db@col@id@w 10%
\db@col@id@end@ %
\db@col@elt@w 123456%
\db@col@elt@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
% Column 11
\db@col@id@w 11%
\db@col@id@end@ %
\db@col@elt@w true%
\db@col@elt@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
% Column 12
\db@col@id@w 12%
\db@col@id@end@ %
\db@col@elt@w female%
\db@col@elt@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
% Column 13
\db@col@id@w 13%
\db@col@id@end@ %
\db@col@elt@w 1989-07-16%
\db@col@elt@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
% End of row 3
\db@row@id@w 3%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 4
\db@row@elt@w %
\db@row@id@w 4%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 4%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Jos\'e%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Arara%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w \@dtlnovalue %
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w Nenhuma Rua%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% Column 6
\db@col@id@w 6%
\db@col@id@end@ %
\db@col@elt@w \@dtlnovalue %
\db@col@elt@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
% Column 7
\db@col@id@w 7%
\db@col@id@end@ %
\db@col@elt@w S\atilde{}o Paulo%
\db@col@elt@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
% Column 8
\db@col@id@w 8%
\db@col@id@end@ %
\db@col@elt@w \@dtlnovalue %
\db@col@elt@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
% Column 9
\db@col@id@w 9%
\db@col@id@end@ %
\db@col@elt@w br%
\db@col@elt@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
% Column 10
\db@col@id@w 10%
\db@col@id@end@ %
\db@col@elt@w 123457%
\db@col@elt@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
% Column 11
\db@col@id@w 11%
\db@col@id@end@ %
\db@col@elt@w true%
\db@col@elt@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
% Column 12
\db@col@id@w 12%
\db@col@id@end@ %
\db@col@elt@w male%
\db@col@elt@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
% Column 13
\db@col@id@w 13%
\db@col@id@end@ %
\db@col@elt@w 1991-05-30%
\db@col@elt@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
% End of row 4
\db@row@id@w 4%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 5
\db@row@elt@w %
\db@row@id@w 5%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 5%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Dickie%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Duck%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w Mr%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 1 The Street%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% Column 6
\db@col@id@w 6%
\db@col@id@end@ %
\db@col@elt@w Another Village%
\db@col@elt@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
% Column 7
\db@col@id@w 7%
\db@col@id@end@ %
\db@col@elt@w Some City%
\db@col@elt@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
% Column 8
\db@col@id@w 8%
\db@col@id@end@ %
\db@col@elt@w Imagineshire%
\db@col@elt@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
% Column 9
\db@col@id@w 9%
\db@col@id@end@ %
\db@col@elt@w gb%
\db@col@elt@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
% Column 10
\db@col@id@w 10%
\db@col@id@end@ %
\db@col@elt@w YZ1 2AB%
\db@col@elt@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
% Column 11
\db@col@id@w 11%
\db@col@id@end@ %
\db@col@elt@w false%
\db@col@elt@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
% Column 12
\db@col@id@w 12%
\db@col@id@end@ %
\db@col@elt@w male%
\db@col@elt@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
% Column 13
\db@col@id@w 13%
\db@col@id@end@ %
\db@col@elt@w 1952-11-25%
\db@col@elt@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
% End of row 5
\db@row@id@w 5%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 6
\db@row@elt@w %
\db@row@id@w 6%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w 6%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w Fred%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w Canary%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% Column 4
\db@col@id@w 4%
\db@col@id@end@ %
\db@col@elt@w Mr%
\db@col@elt@end@ %
\db@col@id@w 4%
\db@col@id@end@ %
% Column 5
\db@col@id@w 5%
\db@col@id@end@ %
\db@col@elt@w 24 The Street%
\db@col@elt@end@ %
\db@col@id@w 5%
\db@col@id@end@ %
% Column 6
\db@col@id@w 6%
\db@col@id@end@ %
\db@col@elt@w Some Village%
\db@col@elt@end@ %
\db@col@id@w 6%
\db@col@id@end@ %
% Column 7
\db@col@id@w 7%
\db@col@id@end@ %
\db@col@elt@w Some Town%
\db@col@elt@end@ %
\db@col@id@w 7%
\db@col@id@end@ %
% Column 8
\db@col@id@w 8%
\db@col@id@end@ %
\db@col@elt@w Noshire%
\db@col@elt@end@ %
\db@col@id@w 8%
\db@col@id@end@ %
% Column 9
\db@col@id@w 9%
\db@col@id@end@ %
\db@col@elt@w gb%
\db@col@elt@end@ %
\db@col@id@w 9%
\db@col@id@end@ %
% Column 10
\db@col@id@w 10%
\db@col@id@end@ %
\db@col@elt@w AB1 2YZ%
\db@col@elt@end@ %
\db@col@id@w 10%
\db@col@id@end@ %
% Column 11
\db@col@id@w 11%
\db@col@id@end@ %
\db@col@elt@w true%
\db@col@elt@end@ %
\db@col@id@w 11%
\db@col@id@end@ %
% Column 12
\db@col@id@w 12%
\db@col@id@end@ %
\db@col@elt@w male%
\db@col@elt@end@ %
\db@col@id@w 12%
\db@col@id@end@ %
% Column 13
\db@col@id@w 13%
\db@col@id@end@ %
\db@col@elt@w 1967-08-04%
\db@col@elt@end@ %
\db@col@id@w 13%
\db@col@id@end@ %
% End of row 6
\db@row@id@w 6%
\db@row@id@end@ %
\db@row@elt@end@ %
}%
\expandafter\global
 \expandafter\newcount\csname dtlrows@people\endcsname
\expandafter\global
 \csname dtlrows@people\endcsname=6\relax
\expandafter\global
 \expandafter\newcount\csname dtlcols@people\endcsname
\expandafter\global
 \csname dtlcols@people\endcsname=13\relax
\expandafter
 \gdef\csname dtl@ci@people@id\endcsname{1}%
\expandafter
 \gdef\csname dtl@ci@people@forenames\endcsname{2}%
\expandafter
 \gdef\csname dtl@ci@people@surname\endcsname{3}%
\expandafter
 \gdef\csname dtl@ci@people@title\endcsname{4}%
\expandafter
 \gdef\csname dtl@ci@people@address1\endcsname{5}%
\expandafter
 \gdef\csname dtl@ci@people@address2\endcsname{6}%
\expandafter
 \gdef\csname dtl@ci@people@town\endcsname{7}%
\expandafter
 \gdef\csname dtl@ci@people@county\endcsname{8}%
\expandafter
 \gdef\csname dtl@ci@people@country\endcsname{9}%
\expandafter
 \gdef\csname dtl@ci@people@postcode\endcsname{10}%
\expandafter
 \gdef\csname dtl@ci@people@subscribed\endcsname{11}%
\expandafter
 \gdef\csname dtl@ci@people@gender\endcsname{12}%
\expandafter
 \gdef\csname dtl@ci@people@dob\endcsname{13}%

\def\people{people}

\expandafter\global\expandafter
\newtoks\csname dtlkeys@xlsproducts\endcsname
\expandafter\global
 \csname dtlkeys@xlsproducts\endcsname={%
%
% header block for column 1
\db@plist@elt@w %
\db@col@id@w 1%
\db@col@id@end@ %
\db@key@id@w Product%
\db@key@id@end@ %
\db@type@id@w 0%
\db@type@id@end@ %
\db@header@id@w Product%
\db@header@id@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 2
\db@plist@elt@w %
\db@col@id@w 2%
\db@col@id@end@ %
\db@key@id@w Price (ex VAT)%
\db@key@id@end@ %
\db@type@id@w 2%
\db@type@id@end@ %
\db@header@id@w Price (ex VAT)%
\db@header@id@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
\db@plist@elt@end@ %
% header block for column 3
\db@plist@elt@w %
\db@col@id@w 3%
\db@col@id@end@ %
\db@key@id@w Price (inc VAT)%
\db@key@id@end@ %
\db@type@id@w 2%
\db@type@id@end@ %
\db@header@id@w Price (inc VAT)%
\db@header@id@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
\db@plist@elt@end@ %
}%
\expandafter\global\expandafter
\newtoks\csname dtldb@xlsproducts\endcsname
\expandafter\global
\csname dtldb@xlsproducts\endcsname={%
%
% Start of row 1
\db@row@elt@w %
\db@row@id@w 1%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w Ink cartridge%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w 25.0%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w 30.0%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% End of row 1
\db@row@id@w 1%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 2
\db@row@elt@w %
\db@row@id@w 2%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w Mouse mat%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w 12.0%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w 14.399999999999999%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% End of row 2
\db@row@id@w 2%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 3
\db@row@elt@w %
\db@row@id@w 3%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w USB stick%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w 15.0%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w 18.0%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% End of row 3
\db@row@id@w 3%
\db@row@id@end@ %
\db@row@elt@end@ %
% Start of row 4
\db@row@elt@w %
\db@row@id@w 4%
\db@row@id@end@ %
% Column 1
\db@col@id@w 1%
\db@col@id@end@ %
\db@col@elt@w Pen%
\db@col@elt@end@ %
\db@col@id@w 1%
\db@col@id@end@ %
% Column 2
\db@col@id@w 2%
\db@col@id@end@ %
\db@col@elt@w 2.5%
\db@col@elt@end@ %
\db@col@id@w 2%
\db@col@id@end@ %
% Column 3
\db@col@id@w 3%
\db@col@id@end@ %
\db@col@elt@w 3.0%
\db@col@elt@end@ %
\db@col@id@w 3%
\db@col@id@end@ %
% End of row 4
\db@row@id@w 4%
\db@row@id@end@ %
\db@row@elt@end@ %
}%
\expandafter\global
 \expandafter\newcount\csname dtlrows@xlsproducts\endcsname
\expandafter\global
 \csname dtlrows@xlsproducts\endcsname=4\relax
\expandafter\global
 \expandafter\newcount\csname dtlcols@xlsproducts\endcsname
\expandafter\global
 \csname dtlcols@xlsproducts\endcsname=3\relax
\expandafter
 \gdef\csname dtl@ci@xlsproducts@Product\endcsname{1}%
\expandafter
 \gdef\csname dtl@ci@xlsproducts@Price (ex VAT)\endcsname{2}%
\expandafter
 \gdef\csname dtl@ci@xlsproducts@Price (inc VAT)\endcsname{3}%
\def\xlsproducts{xlsproducts}

% Cut down version of countries database:

\DTLnewdb{countries}
\DTLnewrow{countries}
\DTLnewdbentry{countries}{code}{br}
\DTLnewdbentry{countries}{name}{Brazil}
\DTLnewrow{countries}
\DTLnewdbentry{countries}{code}{gb}
\DTLnewdbentry{countries}{name}{United Kingdom}
\DTLnewrow{countries}
\DTLnewdbentry{countries}{code}{us}
\DTLnewdbentry{countries}{name}{United States of America}

% Some of the image generation by LaTeX2HTML gets confused
% so find another way to produce them.

% \resultimg{imgfile}{alt text}
\newcommand*{\resultimg}[2]{%
  \begin{result}%
  \input{imagesrc/#1.tex}
  \end{result}
}

\newcommand{\samplecsvlist}{Parrot,Canary,Zebra,Arara,Duck}
\newcommand{\samplecsvlistwithspaces}{ Parrot , Canary , Zebra , Arara , Duck }

\newcommand{\samplecsvlistwithempties}{,Parrot,Canary,Zebra,,Arara,Duck,}

% Hide these from LaTeX2HTML to stop it from getting confused.

\newcommand*{\semicolnamelist}{%
  \renewcommand*{\surnamesep}{\renewcommand*{\surnamesep}{; }}% 
  \DTLforeach*{people}{\Surname=surname}{\surnamesep\Surname}.
}

\newcommand*{\periodnamelist}{%
 \def\do##1{##1. }%
 \expandafter\docsvlist\expandafter{\samplecsvlist}%
}

\newcommand*{\semicolonnamecsvlist}{%
  \renewcommand{\surnamesep}{%
    \renewcommand{\surnamesep}{; }%
  }%
  \renewcommand\do[1]{\surnamesep ##1}%
  \expandafter\docsvlist\expandafter{\samplecsvlist}.%
}

% etextools and etoolbox conflict, so fake this
% (They have different definitions of \forlistloop
% which conflicts with \dolistloop) 
\newcommand*{\periodnamelistcsvloop}{%
 \def\do##1{##1. }%
 \expandafter\docsvlist\expandafter{\samplecsvlist}%
}

% etextools and etoolbox conflict, so fake this
\newcommand*{\periodnamelistcsvloopwithspaces}{%
 \def\tmp{ }%
 \def\do##1{``\tmp##1''. \let\tmp\empty}%
 \expandafter\docsvlist\expandafter{\samplecsvlistwithspaces}%
}

\newcommand*{\doatforexample}{%
  \@for\thisitem:=\samplecsvlist\do{\thisitem. }%
}

\newcommand*{\quotenamelistwithspaces}{%
 \def\do##1{``##1''. }%
 \expandafter\docsvlist\expandafter{\samplecsvlistwithspaces}%
}

\newcommand*{\doatforexamplewithspaces}{%
  \@for\thisitem:=\samplecsvlistwithspaces\do{``\thisitem''. }%
}

\newcommand{\emptyitemsexample}{%
  \def\do##1{``##1''. }%
  \expandafter\docsvlist\expandafter{\samplecsvlistwithempties}%

% etextools and etoolbox conflict, so fake \csvloop
  \expandafter\docsvlist\expandafter{\samplecsvlistwithempties}%

  \foreach \thisitem in \samplecsvlistwithempties {``\thisitem''. }

  \@for\thisitem:=\samplecsvlistwithempties\do{``\thisitem''. }%
}

\newcommand{\dolistloopexample}{%
 \newcommand{\mylist}{}%
 \listadd{\mylist}{Parrot}%
 \listadd{\mylist}{Canary}%
 \listadd{\mylist}{Zebra}%
 \listadd{\mylist}{Arara}%
 \listadd{\mylist}{Duck}%
 \renewcommand\do[1]{##1. }%
 \dolistloop{\mylist}%
}

\newcommand{\etoolboxlistexercise}{%
 \def\titlelist{}%
 \DTLforeach*{books}{\Title=title}%
 {%
  \xifinlist{\Title}{\titlelist}{}{\listeadd{\titlelist}{\Title}}%
 }%
 \def\do##1{##1\par}%
 \dolistloop{\titlelist}%
}

\newcommand*{\dtlifinlistexample}{%
  \let\mylist\samplecsvlist
  Parrot
  \DTLifinlist{Parrot}{\mylist}{is}{isn't}
  in the list.
  Number of items in the list:
  \DTLnumitemsinlist{\mylist}{\numitems}\numitems.
}

\newcommand*{\surnamesep}{}%
\newcommand*{\lastsurname}{}%
\newcommand*{\prelastsurname}{}%

\newcommand*{\displaynames}[1]{%
  \renewcommand*{\surnamesep}{}%
  \renewcommand*{\lastsurname}{}%
  \renewcommand*{\prelastsurname}{}%
  \renewcommand*{\do}[1]{%
    \surnamesep
    \lastsurname
    \renewcommand{\lastsurname}{%
      \renewcommand{\surnamesep}{, }%
      \renewcommand{\prelastsurname}{ and }%
      ##1%
    }}%
  \docsvlist{#1}%
  \prelastsurname \lastsurname
}

\newcommand{\subscribedtrueexample}{%
  \newcommand{\Subscribed}{true}%
  \ifdefstring{\Subscribed}{true}{Yes}{No}%
}

\newcommand{\subscribedoneexample}{%
  \newcommand{\Subscribed}{1}%
  \ifdefstring{\Subscribed}{1}{Yes}{No}%
}

\newcommand*{\xpillustI}{%
\begin{flushleft}
\begin{tabular}{@{}c@{ }c@{ }c}
\glsni{expandafter}&\glsni{docsvlist}&\glsni{expandafter}\\
 & $\uparrow$ & $\uparrow$\\
 & \meta{token 1} & \meta{token 2}
\end{tabular}
\end{flushleft}
}

\newcommand*{\xpillustII}{%
\begin{flushleft}
\begin{tabular}{@{}c@{ }l@{}c}
\glsni{expandafter} &\glsni{leftbracechar}&\cmdname{mylist}\\
 & \llap{\meta{token 1}\raisebox{1ex}{$\nearrow$}} &
\rlap{\raisebox{1ex}{$\nwarrow$}\meta{token 2}}
\end{tabular}
\end{flushleft}
}

\newcommand*{\circledone}{\ding{172}}
\newcommand*{\circledtwo}{\ding{173}}
\newcommand*{\circledthree}{\ding{174}}
\newcommand*{\circledfour}{\ding{175}}
\newcommand*{\circledfive}{\ding{176}}

\newcommand*{\decorativeopenquote}{\ding{123}}

\newcommand*{\ifcsbool}[3]{%
  \ifboolexpr{
    test{\ifdefstring{#1}{true}} or test{\ifdefstring{#1}{1}}%
  }%
  {#2}{#3}%
}

\renewcommand*{\enumerate}{%
\ifnum \@enumdepth > 5
  \@toodeep \else
  \advance\@enumdepth\@ne
  \edef\@enumctr{enum\romannumeral\the\@enumdepth}%
  \expandafter\list\csname label\@enumctr\endcsname{%
     \usecounter\@enumctr
     \def\makelabel##1{\hss\llap{##1}}%
   }%
\fi
}

\newcounter{enumv}[enumiv]
\newcounter{enumvi}[enumv]
\newcommand*{\labelenumv}{\theenumv.}
\newcommand*{\labelenumvi}{\theenumvi.}

\newcounter{para}
\newcommand*{\numberedparagraph}{%
  \refstepcounter{para}\thepara. }

\newcount\mycount

\newcount\julianday
\newcount\dayofweek
\newcommand*\printdate[1]{%
  \pgfcalendardatetojulian{#1}{\julianday}%
  \pgfcalendarjuliantodate{\julianday}{\thisyear}{\thismonth}{\thisday}%
  \pgfcalendarjuliantoweekday{\julianday}{\dayofweek}%
  % Now display the date:
  \datefmt{\thisyear}{\thismonth}{\thisday}{\dayofweek}%
}

\newcommand*\datefmt[4]{%
  \pgfcalendarweekdayname{#4}\space
  \ord{#3}~\pgfcalendarmonthname{#2} \number#1%
}

\newcommand*{\ord}[1]{%
  \number#1%
  \ifcase#1\or st\or nd\or rd\or th\or th\or th\or th\or th\or
  th\or th\or th\or th\or th\or th\or th\or th\or th\or th\or
  th\or th\or st\or nd\or rd\or th\or th\or th\or th\or th\or
  th\or th\or st\fi
}

\newcommand*{\agesexample}{%
\pgfcalendardatetojulian{\year-\month-\day}{\julianday}%
\edef\juliantoday{\number\julianday}%
\begin{tabular}{lc}
\bfseries Name & \bfseries Age%
\DTLforeach*{people}%
 {\Forenames=forenames,\Surname=surname,\DoB=dob}%
 {%
    \\\Forenames\ \Surname &
    % Compute the Julian day number for the date of birth
    \pgfcalendardatetojulian{\DoB}{\julianday}%
    % Compute \age = (\juliantoday - \julianday)/365
    \mycount=\juliantoday
    \advance\mycount by -\julianday
    \divide\mycount by 365\relax
    \number\mycount
 }%
\end{tabular}%
}

\def\parsepdfdatetime#1:#2#3#4#5#6#7#8#9{%
  \parsepdftime
}
\def\parsepdftime#1#2#3#4#5#6#7\endparsepdfdatetime{%
  \def\thehour{#1#2}%
  \def\theminute{#3#4}%
  \def\thesecond{#5#6}%
  \ifstrequal{#7}{Z}%
  {%
    \def\thetimezone{+00:00}%
  }%
  {%
    \parsepdftimezone#7%
  }%
}

\def\parsepdftimezone#1'#2'{%
  \def\thetimezone{#1:#2}%
}

\newcommand*{\timefmt}[4]{%
  #1:#2%
  \ifstrempty{#3}% test for empty 3rd argument
  {}% no seconds specified
  {:#3}% seconds
  \timezonefmt{#4}% time zone
}

\newcommand*{\timezonefmt}[1]{%
  \edef\thistimezone{#1}%
  \ifdefempty{\thistimezone}%
  {}% empty argument
  {%
     \expandafter\@timezonefmt\thistimezone\@endtimezonefmt%
  }%
}

\def\@timezonefmt#1:#2\@endtimezonefmt{%
  \ifnum#2=0\relax
    \ifnum#1=0\relax
      Z%
    \else
      #1%
    \fi
  \else
    #1:#2%
  \fi
}

\expandafter\parsepdfdatetime\pdfcreationdate\endparsepdfdatetime
\let\pdfhour\thehour
\let\pdfminute\theminute
\let\pdfsecond\thesecond
\let\pdftimezone\thetimezone

\newcommand*{\pdfnowtime}{\timefmt{\pdfhour}{\pdfminute}{\pdfsecond}{\pdftimezone}}

% exam.cls slanted half:
\newcommand*{\examhalf}{%
 \begin{math}%
 \raise 0.6ex\hbox{$\scriptstyle 1$}%
 \kern -.2em/\kern -.2em
 \raise -0.5ex\hbox {$\scriptstyle 2$}%
 \end{math}%
}

% simulate exam.cls questions environment:
\newenvironment{examquestions}%
{%
  \begin{list}%
  {\arabic{enumi}.}%
  {\usecounter{enumi}%
    \settowidth{\leftmargin}{10.\hskip\labelsep}%
    \labelwidth\leftmargin
    \advance\labelwidth -\labelsep
    \partopsep =0pt\relax
  }%
}%
{\end{list}}

\newcommand*{\exam@questionpoints}{}

\newcommand*{\examquestion}[1][]{%
 \ifblank{#1}%
 {%
   \let\exam@questionpoints\@empty
 }%
 {%
   \ifstrequal{#1}{1}%
   {%
     \renewcommand*{\exam@questionpoints}{\space(#1 point)\space}%
   }%
   {%
     \ifstrequal{#1}{\examhalf}%
     {%
       \renewcommand*{\exam@questionpoints}{\space(#1 point)\space}%
     }%
     {%
       \renewcommand*{\exam@questionpoints}{\space(#1 points)\space}%
     }%
   }%
 }%
 \refstepcounter{enumi}%
 \@item[\@itemlabel]\exam@questionpoints
}

\newenvironment{examparts}%
{\begin{list}%
 {(\alph{enumii})}%
 {\usecounter{enumii}%
 \def\makelabel##1{\hss\llap{##1}}%
 \settowidth{\leftmargin}{(m)\hskip\labelsep}%
 \labelwidth\leftmargin
 \advance\labelwidth -\labelsep
 \topsep =0pt
 \partopsep =0pt}}%
{\end{list}}

\newcommand*{\exam@partpoints}{}

\newcommand*{\exampart}[1][]{%
 \ifblank{#1}%
 {%
   \let\exam@partpoints\@empty
 }%
 {%
   \ifstrequal{#1}{1}%
   {%
     \renewcommand*{\exam@partpoints}{\space(#1 point)\space}%
   }%
   {%
     \ifstrequal{#1}{\examhalf}%
     {%
       \renewcommand*{\exam@partpoints}{\space(#1 point)\space}%
     }%
     {%
       \renewcommand*{\exam@partpoints}{\space(#1 points)\space}%
     }%
   }%
 }%
 \refstepcounter{enumii}%
 \@item[\@itemlabel]\exam@partpoints
}

\newenvironment{examchoices}%
{%
 \begin{list}%
 {\Alph{enumii}.}%
 {\usecounter{enumii}%
 \def\makelabel##1{\hss\llap{##1}}%
 \settowidth{\leftmargin}{M.\hskip\labelsep\hskip 2.5em}%
 \labelwidth\leftmargin
 \advance\labelwidth -\labelsep
 \topsep =0pt
 \partopsep =0pt}%
 \def\examchoice{\item}%
}%
{\end{list}}

\newcommand*{\mirror}[1]{%
  \ifbookorother
  {\url{http://mirrors.ctan.org/#1}}
  {\htmladdnormallink{CTAN Mirror}{http://mirrors.ctan.org/#1}}%
}

% latex2html gets a bit confused with \~{a} inside alltt
\DeclareRobustCommand*{\atilde}{\~{a}}

% Can't load ifsym with geometry option or it will clash with
% wasysym so just define the ifsym symbols used in the book
\newcommand*{\BigCircle}{\textifsymbol[ifgeo]{37}}
\newcommand*{\FilledSmallCircle}{\textifsymbol[ifgeo]{117}}

\newcommand*{\tikzchecked}{\includegraphics[height=2ex]{tikzchecked}}

\newcommand*{\tikzunchecked}{\includegraphics[height=2ex]{tikzunchecked}}

\newcommand*{\ecircumacute}{\'{\^{e}}}

\newlength\frameoffset

\newcommand{\cutline}[1][4em]{%
  \par\noindent
  \ifodd\value{page}\relax
    \computeleftedgeodd\frameoffset
    \getflowbounds{\value{thisframe}}%
  \else
    \computeleftedgeeven\frameoffset
    \getflowevenbounds{\value{thisframe}}%
  \fi
  \addtolength{\frameoffset}{-\ffareax}%
  \hspace*{\frameoffset}%
  \rlap{%
   \makebox[\paperwidth][l]{%
     \makebox[#1]{\dotfill}\ding{33}\dotfill
  }}\par
}

\newcommand*{\midtabsep}{%
  \noalign {\ifnum 0=`}\fi
  \vskip .5ex
  \ifnum 0=`{\fi }%
}

\endinput
